
/** 
* @file PmxBaseClass.cpp
* @brief  PMX library source file
* @author Kondo Kagaku Co.,Ltd.
* @author T.Nobuhara Gotanda
* @date 2023/12/13
* @version 1.0.0
* @copyright Kondo Kagaku Co.,Ltd. 2023
*
*/

#include "PmxBaseClass.h"
#include "PmxCRC.h"
#include "DataConvert.h"



/**
 * checkRecv関数は、受信パケットのヘッダー、パケット長、およびコマンドをチェックします。
 * 
 * @param [in] rxBuff 受信したCRCを含む全てのデータ(バイト配列)
 * @param [in] cmd 受信時にチェックするコマンド
 * @param [in] header ヘッダーがデフォルトと違う場合に指定する
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::checkRecv(byte rxBuff[],byte cmd,byte header)
{
    // sync関数で正常なデータ数が来なかったらFalseなので、それ以外をチェック
    
    // ヘッダのチェック
    if(rxBuff[0] != header || rxBuff[1] != header)
    {
        Serial.println("HeaderError");
        return PMX::ComError::ReceiveError;
    }

    //コマンドのチェック
    if(rxBuff[PMX::BuffPter::CMD] != (cmd & 0x7f ))
    {
        Serial.println("cmdError");
        return PMX::ComError::ReceiveError;
    }

    // CRCチェック
    if(false == PmxCrc16::checkCrc16(rxBuff))
    {
        return PMX::ComError::CrcError;
    }

    return PMX::ComError::OK;

}



/**
 * MemREADコマンドでPMXサーボモータからデータを取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] addr データを読み取る先頭のアドレス
 * @param [in] readDataSize 取得するデータサイズ
 * @param [out] rxData 指定されたアドレスから読み取ったデータ配列(bytes型)。通信失敗時にはNoneを返します。
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::MemREAD(byte id, unsigned short addr, int readDataSize, byte rxData[])
{
    int PUT_MAX_DATA_LENGTH = 244;

    if(readDataSize == 0 || readDataSize >= PUT_MAX_DATA_LENGTH)
    {
        return PMX::ComError::FormatError;
    }

    int txSize = 11;    //memREADはLength固定
    int rxSize = 8 + readDataSize;
    
    byte txbuf[txSize];
    byte rxbuf[rxSize];


    txbuf[0] = (byte)(0xFE);           // HEADER
    txbuf[1] = (byte)(0xFE);           // HEADER
    txbuf[2] = (byte)(id);             // ID
    txbuf[3] = (byte)(txSize);        // LENGTH
    txbuf[4] = PMX::SendCmd::MemREAD; //Comannd
    txbuf[5] = (byte)(0x00);           // OPTION
    txbuf[6] = (byte)(addr & 0x00ff);           //先頭アドレス（下位）
    txbuf[7] = (byte)((addr & 0xff00) >> 8);    //先頭アドレス（上位）
    txbuf[8] = (byte)(readDataSize);      // データサイズ

    PmxCrc16::setCrc16(txbuf);

    bool rxFlag= this->synchronize(txbuf, txSize , rxbuf, rxSize);

    this->logOutputPrint(txbuf,txSize);

    if(rxFlag == false)
    {
        //Serial.println("timeout");
        
        //配列データはエラー値を入れておく
        for(int i = 0; i < readDataSize; i++)
        {
            rxData[i] = 0xFF;
        }

        return PMX::ComError::TimeOut;
    }

    this->logOutputPrint(rxbuf,rxSize);

    unsigned short errorFlag = this->checkRecv(rxbuf, PMX::SendCmd::MemREAD);


    if(errorFlag != PMX::ComError::OK)  //ここの部分はステータスを含まないのでOK
    {
        
        return errorFlag;
    }

    unsigned short status = rxbuf[PMX::BuffPter::Status];

    for(int i = 0; i < readDataSize ; i++)
    {
        rxData[i] = rxbuf[PMX::BuffPter::Data + i];
    }

    return status;

}

/**
 *  MemREADで取得したデータをbyte(符号なし1byte)に変換して取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] addr データを読み取る先頭のアドレス
 * @param [out] byteData 指定されたアドレスから読み取ったbyte(符号なし1byte)のデータ。通信失敗時にはNoneを返します。
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::MemREADToByte(byte id, unsigned short addr , byte *byteData)
{
    unsigned short status = this->MemREAD(id, addr, 1, byteData);

    //通信エラーの時は0xFFを返す
    if( (status & PMX::ComError::ErrorMask) != 0x0000 )
    {
        *byteData = PMX::ErrorByteData;
    }

    return status;
}

/**
 * MemREADで取得したデータをint16(符号あり2byte)に変換して取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] addr データを読み取る先頭のアドレス
 * @param [out] int16Data 指定されたアドレスから読み取ったint16(符号あり2byte)のデータ。通信失敗時にはNoneを返します。
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::MemREADToInt16(byte id, unsigned short addr , short *int16Data)
{
    byte reByte[2];
    unsigned short status = this->MemREAD(id, addr, 2, reByte);

    *int16Data = DataConv::bytesToInt16(reByte);

    //通信エラーの時は0x7FFFを入れておく
    if( (status & PMX::ComError::ErrorMask) != 0x0000 )
    {
        *int16Data = (short)PMX::ErrorUint16Data;
    }

    return status;
}

/**
 * MemREADで取得したデータをUint16(符号なし2byte)に変換して取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] addr データを読み取る先頭のアドレス
 * @param [out] uint16Data 指定されたアドレスから読み取ったUint16(符号なし2byte)のデータ。通信失敗時にはNoneを返します。
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::MemREADToUint16(byte id, unsigned short addr , unsigned short *uint16Data)
{
    byte reByte[2];
    unsigned short status = this->MemREAD(id, addr, 2, reByte);

    *uint16Data = DataConv::bytesToInt16(reByte);

    //通信エラーの時は0x7FFFを入れておく
    if( (status & PMX::ComError::ErrorMask) != 0x0000 )
    {
        *uint16Data = (short)PMX::ErrorUint16Data;
    }

    return status;
}


/**
 * MemREADで取得したデータをint32(符号あり4byte)に変換して取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] addr データを読み取る先頭のアドレス
 * @param [out] int32Data 指定されたアドレスから読み取ったint32(符号あり4byte)のデータ。通信失敗時にはNoneを返します。
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::MemREADToInt32(byte id, unsigned short addr , long *int32Data)
{
    byte reByte[2];
    unsigned short status = this->MemREAD(id, addr, 2, reByte);

    *int32Data = DataConv::bytesToInt32(reByte);

    //通信エラーの時は0x7FFFを入れておく
    if( (status & PMX::ComError::ErrorMask) != 0x0000 )
    {
        *int32Data = (short)PMX::ErrorUint32Data;
    }

    return status;
}

/**
 * MemREADで取得したデータをUint32(符号なし4byte)に変換して取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] addr データを読み取る先頭のアドレス
 * @param [out] uint32Data 指定されたアドレスから読み取ったUint32(符号なし4byte)のデータ。通信失敗時にはNoneを返します。
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::MemREADToUint32(byte id, unsigned short addr , unsigned long *uint32Data)
{
    byte reByte[2];
    unsigned short status = this->MemREAD(id, addr, 2, reByte);

    *uint32Data = DataConv::bytesToInt32(reByte);

    //通信エラーの時は0x7FFFを入れておく
    if( (status & PMX::ComError::ErrorMask) != 0x0000 )
    {
        *uint32Data = (short)PMX::ErrorUint32Data;
    }

    return status;
}



/**
 * PMXサーボモータにMemWRITEコマンドを発行し、結果を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] addr データを書き込む先頭アドレス
 * @param [in] txDataArray 書き込むデータのbyteリスト
 * @param [in] txDataSize 書き込むデータサイズ(txDataArrayのサイズ) 
 * @param [in] writeOpt MemWRITEで使用するオプション 0:通常書き込み、1:TorqueOn中の強制書き込み
 * 
 * @return MemWRITEの送受信結果(送信結果+PMXのstatus(Uint16))
 */
unsigned short PmxBace::MemWRITE(byte id, unsigned short addr, byte txDataArray[], int txDataSize, byte writeOpt)
{
    int PUT_MAX_DATA_LENGTH = 245;

    // byte writeDataBuff[2];
    // DataConv::bytesToShort(1000, writeDataBuff);

    // 最大値の判定
    if(txDataSize == 0 || txDataSize >= PUT_MAX_DATA_LENGTH)
    {
        return PMX::ComError::FormatError;
    }

    int txSize = PMX::MinimumLength::Send + 2 + txDataSize;    //memREADはLength固定
    int rxSize = PMX::MinimumLength::Receive;
    
    
    byte txbuf[txSize];
    byte rxbuf[rxSize];

    txbuf[0] = (byte)(0xFE);            // HEADER
    txbuf[1] = (byte)(0xFE);            // HEADER
    txbuf[2] = (byte)(id);              // ID
    txbuf[3] = (byte)(txSize);          // LENGTH
    txbuf[4] = PMX::SendCmd::MemWRITE;  //Comannd
    txbuf[5] = (byte)(writeOpt);        // OPTION
    txbuf[6] = (byte)(addr & 0x00ff);           //先頭アドレス（下位）
    txbuf[7] = (byte)((addr & 0xff00) >> 8);    //先頭アドレス（上位）

    for(int i = 0; i < txDataSize; i++){
        txbuf[8+i] = (byte)(txDataArray[i]); //書込データ
    }

    PmxCrc16::setCrc16(txbuf);

    bool rxFlag= this->synchronize(txbuf, txSize , rxbuf, rxSize);

    this->logOutputPrint(txbuf,txSize);

    if(rxFlag == false)
    {
        //Serial.println("timeout");
        return PMX::ComError::TimeOut;
    }

    this->logOutputPrint(rxbuf,rxSize);

    unsigned short errorFlag = this->checkRecv(rxbuf, PMX::SendCmd::MemWRITE);


    if(errorFlag != PMX::ComError::OK)  //ここの部分はステータスを含まないのでOK
    {
        return errorFlag;
    }

    unsigned short status = rxbuf[PMX::BuffPter::Status];

    return status;
}


/**
 * MemWRITEでbyte(符号なし1byte)のデータを書き込みます。
        その結果のステータス情報を返信します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] addr データを書き込む先頭アドレス
 * @param [in] byteData byte(符号なし1byte)の書きこみデータ
 * @param [in] writeOpt MemWRITEで使用するオプション 0:通常書き込み、1:TorqueOn中の強制書き込み
 * 
 * @return MemWRITEの送受信結果(送信結果+PMXのstatus(Uint16))
 */
unsigned short PmxBace::MemWRITEToByte(byte id, unsigned short addr, byte byteData, byte writeOpt)
{
    unsigned short status = this->MemWRITE(id, addr, &byteData, 1, writeOpt);

    return status;
}


/**
 * MemWRITEでint16(符号あり2byte)のデータを書き込みます。
        その結果のステータス情報を返信します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] addr データを書き込む先頭アドレス
 * @param [in] int16Data int16(符号あり2byte)の書きこみデータ
 * @param [in] writeOpt MemWRITEで使用するオプション 0:通常書き込み、1:TorqueOn中の強制書き込み
 * 
 * @return MemWRITEの送受信結果(送信結果+PMXのstatus(Uint16))
 */
unsigned short PmxBace::MemWRITEToInt16(byte id, unsigned short addr, short int16Data, byte writeOpt)
{
    byte txDataArray[2];
    DataConv::int16ToBytes(int16Data, txDataArray);
    unsigned short status = this->MemWRITE(id, addr, txDataArray, 2, writeOpt);

    return status;
}


/**
 *  MemWRITEでint16(符号なし2byte)のデータを書き込みます。
        その結果のステータス情報を返信します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] addr データを書き込む先頭アドレス
 * @param [in] uint16Data int16(符号なし2byte)の書きこみデータ
 * @param [in] writeOpt MemWRITEで使用するオプション 0:通常書き込み、1:TorqueOn中の強制書き込み
 * 
 * @return MemWRITEの送受信結果(送信結果+PMXのstatus(Uint16))
 */
unsigned short PmxBace::MemWRITEToUint16(byte id, unsigned short addr, unsigned short uint16Data, byte writeOpt)
{
    byte txDataArray[2];
    DataConv::uint16ToBytes(uint16Data, txDataArray);
    unsigned short status = this->MemWRITE(id, addr, txDataArray, 2, writeOpt);

    return status;
}


/**
 * MemWRITEでint32(符号あり4byte)のデータを書き込みます。
        その結果のステータス情報を返信します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] addr データを書き込む先頭アドレス
 * @param [in] int32Data Uint32(符号あり4byte)の書きこみデータ
 * @param [in] writeOpt MemWRITEで使用するオプション 0:通常書き込み、1:TorqueOn中の強制書き込み
 * 
 * @return MemWRITEの送受信結果(送信結果+PMXのstatus(Uint16))
 */
unsigned short PmxBace::MemWRITEToInt32(byte id, unsigned short addr, long int32Data, byte writeOpt)
{
    byte txDataArray[4];
    DataConv::int32ToBytes(int32Data, txDataArray);
    unsigned short status = this->MemWRITE(id, addr, txDataArray, 4, writeOpt);

    return status;
}


/**
 * MemWRITEでUint32(符号なし4byte)のデータを書き込みます。
        その結果のステータス情報を返信します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] addr データを書き込む先頭アドレス
 * @param [in] uint32Data Uint32(符号なし4byte)の書きこみデータ
 * @param [in] writeOpt MemWRITEで使用するオプション 0:通常書き込み、1:TorqueOn中の強制書き込み
 * 
 * @return MemWRITEの送受信結果(送信結果+PMXのstatus(Uint16))
 */
unsigned short PmxBace::MemWRITEToUint32(byte id, unsigned short addr, unsigned long uint32Data, byte writeOpt)
{
    byte txDataArray[4];
    DataConv::uint32ToBytes(uint32Data, txDataArray);
    unsigned short status = this->MemWRITE(id, addr, txDataArray, 4, writeOpt);

    return status;
}


/**
 * PMXサーボモータのLOADコマンドを発行し、ステータス結果を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * 
 * @return LOADの送受信結果(送信結果+PMXのstatus(Uint16))
 */
unsigned short PmxBace::LOAD(byte id)
{

    int txSize = PMX::MinimumLength::Send;    
    int rxSize = PMX::MinimumLength::Receive;
    
    byte txbuf[txSize];
    byte rxbuf[rxSize];

    txbuf[0] = (byte)(0xFE);            // HEADER
    txbuf[1] = (byte)(0xFE);            // HEADER
    txbuf[2] = (byte)(id);              // ID
    txbuf[3] = (byte)(txSize);          // LENGTH
    txbuf[4] = PMX::SendCmd::LOAD;      //Comannd
    txbuf[5] = (byte)(0x00);            // OPTION

    PmxCrc16::setCrc16(txbuf);

    bool rxFlag= this->synchronize(txbuf, txSize , rxbuf, rxSize);

    this->logOutputPrint(txbuf,txSize);

    if(rxFlag == false)
    {
        Serial.println("timeout");
        return PMX::ComError::TimeOut;
    }

    this->logOutputPrint(rxbuf,rxSize);

    unsigned short errorFlag = this->checkRecv(rxbuf, PMX::SendCmd::LOAD);


    if(errorFlag != PMX::ComError::OK)  //ここの部分はステータスを含まないのでOK
    {
        return errorFlag;
    }

    unsigned short status = rxbuf[PMX::BuffPter::Status];

    return status;
}


/**
 * PMXサーボモータのSAVEコマンドを発行し、ステータス結果を取得します。
 * 
 * @param [in] id サーボモータのID番号
 * 
 * @return SAVEコマンドの送信結果(送信結果+PMXのstatus(Uint16))
 */
unsigned short PmxBace::SAVE(byte id)
{

    int txSize = PMX::MinimumLength::Send;    
    int rxSize = PMX::MinimumLength::Receive;
    
    byte txbuf[txSize];
    byte rxbuf[rxSize];

    txbuf[0] = (byte)(0xFE);            // HEADER
    txbuf[1] = (byte)(0xFE);            // HEADER
    txbuf[2] = (byte)(id);              // ID
    txbuf[3] = (byte)(txSize);          // LENGTH
    txbuf[4] = PMX::SendCmd::SAVE;      //Comannd
    txbuf[5] = (byte)(0x00);            // OPTION

    PmxCrc16::setCrc16(txbuf);

    bool rxFlag= this->synchronize(txbuf, txSize , rxbuf, rxSize);

    this->logOutputPrint(txbuf,txSize);

    if(rxFlag == false)
    {
        Serial.println("timeout");
        return PMX::ComError::TimeOut;
    }

    this->logOutputPrint(rxbuf,rxSize);

    unsigned short errorFlag = this->checkRecv(rxbuf, PMX::SendCmd::SAVE);


    if(errorFlag != PMX::ComError::OK)  //ここの部分はステータスを含まないのでOK
    {
        return errorFlag;
    }

    unsigned short status = rxbuf[PMX::BuffPter::Status];

    return status;
}


/**
 * MotorREAD関数を発行し、応答モードに応じてデータを取得します。
 * 
 * @param [in] id サーボモータのID番号
 * @param [in] receiveMode 応答モード。事前に応答モードをMemWrite関数やsetMotorReceive関数で設定する必要があります。
 * @param [out] readMotorData MotorReadで読み取ったデータ配列[位置,速度,電流,トルク,PWM,モータ温度,CPU温度,電圧](合計8項目)
 * @param [in] controlMode 制御モード。デフォルト値は `0x01` です。位置制御モードの時の現在位置を判定するときに使用します。
 * @param [in] torqueSw FreeやTorqueOnの値(PMX::TorqueSwitchTypeを参照)
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::MotorREAD(byte id, byte receiveMode, long readMotorData[8], byte controlMode, byte *torqueSw)
{
    int readDataSize = this->__byteCounter(receiveMode);

    int txSize = PMX::MinimumLength::Send;    //MotorREADはLength固定
    int rxSize = PMX::MinimumLength::Receive + 1 + readDataSize;
    
    byte txbuf[txSize];
    byte rxbuf[rxSize];
    
    //返すデータは初期化しておく
    for(int i = 0;i<8;i++)
    {
        readMotorData[i] = PMX::ErrorUint32Data;
    }

    if(torqueSw != NULL)
    {
        *torqueSw = PMX::TorqueSwitchType::Error;
    }

    txbuf[0] = (byte)(0xFE);           // HEADER
    txbuf[1] = (byte)(0xFE);           // HEADER
    txbuf[2] = (byte)(id);             // ID
    txbuf[3] = (byte)(txSize);        // LENGTH
    txbuf[4] = PMX::SendCmd::MotorREAD; //Comannd
    txbuf[5] = (byte)(0x00);           // OPTION

    PmxCrc16::setCrc16(txbuf);

    //bool rxFlag= this->synchronize(txbuf, txSize , rxbuf, rxSize);

    byte rxNowSize;
    bool rxFlag = this->synchronizeVariableRead(txbuf, txSize , rxbuf, &rxNowSize);

    //Serial.println(rxNowSize);

    this->logOutputPrint(txbuf,txSize);

    if(rxFlag == false)
    {
        return PMX::ComError::TimeOut;
    }

    this->logOutputPrint(rxbuf,rxNowSize);

    unsigned short errorFlag = this->checkRecv(rxbuf, PMX::SendCmd::MotorREAD);
    if(errorFlag != PMX::ComError::OK)  //ここの部分はステータスを含まないのでOK
    {
        return errorFlag;
    }


    //トルクスイッチ情報を代入
    if((torqueSw != NULL) && (rxNowSize >= PMX::MinimumLength::Receive))
    {
        *torqueSw = rxbuf[PMX::BuffPter::Data];
    }



    //ステータス情報を代入
    unsigned short status = rxbuf[PMX::BuffPter::Status];



    if(rxNowSize != rxSize) //実際受信した情報と予想していた情報が違う
    {
        status += PMX::ComError::MotorREADConvertError;  //受信エラーを含んだ情報を返す
        return  status;
    }

    byte rxMotorData[readDataSize];  //受信したデータをためておくバッファ

    //受信データを抜き出す
    for(int i = 0; i < readDataSize ; i++)
    {
        rxMotorData[i] = rxbuf[PMX::BuffPter::Data + 1 + i];
    }

    //　データの代入
    bool flag = this->__convReceiveMotorData(receiveMode, rxMotorData, readDataSize, readMotorData, controlMode);
    if(flag == false)   //何かしら失敗したらエラーを付加して抜ける
    {
        status += PMX::ComError::MotorREADConvertError;  //受信エラーを含んだ情報を返す
        return  status;
    }

    return status;
}


//制御モード、応答モードからByte数をカウントする
/**
 * 制御モード、応答モードを基に必要とするデータが合計何byteになるのか計算をします。
 * 
 * @param [in] val 応答モードもしくは制御モード
 * 
 * @return 制御モードや応答モードに必要とするデータ(byte)数
 */
unsigned int PmxBace::__byteCounter(byte val)
{
    
    //制御モード、応答モードを基に必要とするデータが合計何byteになるのか計算をします。
    //@param val 応答モードもしくは制御モード
    //@return 制御モードや応答モードに必要とするデータ(byte)数
    
    int cnt = 0;

    for(int i = 0; i < 8; i++){
        int check = val & 0x01;
        if(check == 1){cnt++;}
        val = val >> 1;
    }
    
    //1data2byteなので、2倍の数値を返す
    return cnt * 2;
}


/**
 * 受信モードに基づいてMotorWrite/Readで送られて来たデータを変換します。
 * 
 * @param [in] receiveMode 受信モード設定
 * @param [in] receiveBytes 受信されたデータのbyte配列（コマンド等は含まない）
 * @param [in] receiveBytesSize 受信データ (コマンドを除く) を含む、receiveBytes 配列のサイズ。
 * @param [out] motorData 変換されたモーター データを格納する長整数の配列。位置、速度、電流、トルク、PWM、モーター温度、CPU 温度、電圧のデータを順番に保存するため、配列のサイズは 8
 * にする必要があります。
 * @param [in] controlMode 制御モードは、位置データ変換の制御モードを指定するオプションのパラメータです。デフォルト値は 0x01 で、位置制御モードを表します。
 * 
 * @return 受信データを変換したモータデータを返す[位置,速度,電流,トルク,PWM,モータ温度,CPU温度,電圧]
 */
bool PmxBace::__convReceiveMotorData(byte receiveMode, byte receiveBytes[], byte receiveBytesSize, long motorData[], byte controlMode)
{
    //受信モードに基づいてMotorWrite/Readで送られて来たデータを変換します。
    
    //@param receiveMode 受信モード設定
    //@param returnDataBytes 受信されたデータのbyte配列（コマンド等は含まない）
    //@param controlMode 制御モード(位置制御のみ符号ありで変換するため 
    //@return 受信データを変換したモータデータを返す[位置,速度,電流,トルク,PWM,モータ温度,CPU温度,電圧]
    //@attention データがない場合はリストの中身の各データがNoneになります
    
    //dataListの定義[位置,速度,電流,トルク,PWM,モータ温度,CPU温度,電圧]
    //int reDataList[] = [None, None, None, None, None, None, None, None];

    for(int i = 0;i<8;i++)
    {
        motorData[i] = PMX::ErrorUint32Data;
    }

    //データbyte数と変換するデータ数が違っていたら空リストを返す
    if (receiveBytesSize != (__byteCounter(receiveMode)))
    {
        return false;
    }

    int arrayCount = 0;  // データを数える(bytes配列の場所を決めるのに使用)
    
    //
    //  位置データ
    //
    if((receiveMode & PMX::ReceiveDataOption::Position) == PMX::ReceiveDataOption::Position)
    {

        //位置制御モードの時
        if((controlMode & PMX::ControlMode::Position) == PMX::ControlMode::Position)
        {
            motorData[0] = (long)DataConv::bytesToInt16(&(receiveBytes[arrayCount]));
        } 
        else
        {
            motorData[0] = (long)DataConv::bytesToUint16(&(receiveBytes[arrayCount]));
        } 

        arrayCount += 2;
        //listCount++;
    }

    //
    //  速度データ
    //
    if((receiveMode & PMX::ReceiveDataOption::Speed) == PMX::ReceiveDataOption::Speed)
    {
        motorData[1] = (long)DataConv::bytesToInt16(&(receiveBytes[arrayCount]));
        arrayCount += 2;
    }

    //
    //  電流データ
    //
    if((receiveMode & PMX::ReceiveDataOption::Current) == PMX::ReceiveDataOption::Current)
    {
        motorData[2] = (long)DataConv::bytesToInt16(&(receiveBytes[arrayCount]));
        arrayCount += 2;
    }

    //
    //  トルクデータ
    //
    if((receiveMode & PMX::ReceiveDataOption::Torque) == PMX::ReceiveDataOption::Torque)
    {
        motorData[3] = (long)DataConv::bytesToInt16(&(receiveBytes[arrayCount]));
        arrayCount += 2;
    }

    //
    //  PWMデータ
    //
    if((receiveMode & PMX::ReceiveDataOption::Pwm) == PMX::ReceiveDataOption::Pwm)
    {
        motorData[4] = (long)DataConv::bytesToInt16(&(receiveBytes[arrayCount]));
        arrayCount += 2;
    }

    //
    //  モータ温度データ
    //
    if((receiveMode & PMX::ReceiveDataOption::MotorTemp) == PMX::ReceiveDataOption::MotorTemp)
    {
        motorData[5] = (long)DataConv::bytesToInt16(&(receiveBytes[arrayCount]));
        arrayCount += 2;
    }

    //
    //  CPU温度データ
    //
    if((receiveMode & PMX::ReceiveDataOption::CpuTemp) == PMX::ReceiveDataOption::CpuTemp)
    {
        motorData[6] = (long)DataConv::bytesToInt16(&(receiveBytes[arrayCount]));
        arrayCount += 2;
    }

    //
    //  電圧データ
    //
    if((receiveMode & PMX::ReceiveDataOption::Voltage) == PMX::ReceiveDataOption::Voltage)
    {
        motorData[7] = (long)DataConv::bytesToUint16(&(receiveBytes[arrayCount]));
        arrayCount += 2;
    }


    return true;
}


/**
 * MotorWRITE 機能は、モータの制御やトルクのオン/オフを切り替えるコマンドを送信し、モーターからステータスとデータを受信します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] toruqeOnSw 「toruqeOnSw」パラメータは、モーターのトルク オン/オフ スイッチを表すバイトです。
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::MotorWRITE(byte id, byte toruqeOnSw)
{
    byte reDummyReMode = PMX::ReceiveDataOption::NoReturn;
    long dummyDatas[8];

    return this->MotorWRITE(id, toruqeOnSw, reDummyReMode, dummyDatas);
}


/**
 * MotorWRITE 機能は、モータの制御やトルクのオン/オフを切り替えるコマンドを送信し、モーターからステータスとデータを受信します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] toruqeOnSw パラメータ「toruqeOnSw」はトルクスイッチの状態を表します。次のいずれかの値を指定できます。
 * @param [in] receiveMode 応答モード。返ってきたデータを変換するときに使用します。
 * @param [in] receiveData 受信したデータを格納するために使用される長整数の配列。配列のサイズは 8 です。
 * @param [in] controlMode モータの制御モード。応答モードで現在位置を取得する時のみに使用します
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::MotorWRITE(byte id, byte toruqeOnSw, byte receiveMode, long receiveData[8], byte controlMode)
{
    //
    //  トルクON等トルクスイッチをいじる関数
    //



    //返すデータは初期化しておく
    if(receiveData != NULL)
    {    
        for(int i = 0;i<8;i++)
        {
            receiveData[i] = PMX::ErrorUint32Data;
        }
    }


    // トルクスイッチが規定値にない
    if( ! (toruqeOnSw == PMX::TorqueSwitchType::TorqueOn ||
        toruqeOnSw == PMX::TorqueSwitchType::Free ||
        toruqeOnSw == PMX::TorqueSwitchType::Brake ||
        toruqeOnSw == PMX::TorqueSwitchType::Hold ) )
    {
        return PMX::ComError::FormatError;
    }

    int readDataSize = this->__byteCounter(receiveMode);;
    

    int txSize = PMX::MinimumLength::Send;    //MotorREADはLength固定
    int rxSize = PMX::MinimumLength::Receive + 1 + readDataSize;
    
    byte txbuf[txSize]; //トルクスイッチ時は送信データ固定
    byte rxbuf[26]; //motorwriteの最大値

    
    txbuf[0] = (byte)(0xFE);           // HEADER
    txbuf[1] = (byte)(0xFE);           // HEADER
    txbuf[2] = (byte)(id);             // ID
    txbuf[3] = (byte)(txSize);        // LENGTH
    txbuf[4] = PMX::SendCmd::MotorWRITE; //Comannd
    txbuf[5] = (byte)(toruqeOnSw);           // OPTION

    PmxCrc16::setCrc16(txbuf);

    
    byte rxNowSize;
    bool rxFlag = this->synchronizeVariableRead(txbuf, txSize , rxbuf, &rxNowSize);

    //Serial.println(rxNowSize);

    this->logOutputPrint(txbuf,txSize);

    //データが来てたかどうか
    if(rxFlag == false)
    {
        return PMX::ComError::TimeOut;
    }

    //データの判定
    unsigned short errorFlag = this->checkRecv(rxbuf, PMX::SendCmd::MotorWRITE);
    if(errorFlag != PMX::ComError::OK)  //ここの部分はステータスを含まないのでOK
    {
        return errorFlag;
    }

    this->logOutputPrint(rxbuf,rxNowSize);

    //トルクスイッチ情報を代入
    // if((torqueSw != NULL) && (rxNowSize >= PMX::MinimumLength::Receive))
    // {
    //     *torqueSw = rxbuf[PMX::BuffPter::Data];
    // }

    //ステータス情報を代入
    unsigned short status = rxbuf[PMX::BuffPter::Status];

    

    //データを返さなくていい時はtrueで返す
    if(receiveMode == PMX::ReceiveDataOption::NoReturn)
    {
        return status;
    }

    // ↓↓↓↓↓↓ここからはReceiveDataの処理↓↓↓↓↓

    if(rxNowSize != rxSize) //実際受信した情報と予想していた情報が違う
    {
        status += PMX::ComError::ReceiveError;  //受信エラーを含んだ情報を返す
        return  status;
    }


    byte rxMotorData[readDataSize];  //受信したデータをためておくバッファ

    //受信データを抜き出す
    for(int i = 0; i < readDataSize ; i++)
    {
        rxMotorData[i] = rxbuf[PMX::BuffPter::Data + 1 + i];
    }

    //　データの代入
    bool flag = this->__convReceiveMotorData(receiveMode, rxMotorData, readDataSize, receiveData, controlMode);
    if(flag == false)   //何かしら失敗したらエラーを付加して抜ける
    {
        status += PMX::ComError::MotorREADConvertError;  //受信エラーを含んだ情報を返す
        return  status;
    }





    return status;

}

/**
 * MotorWRITE 機能は、モータの制御やトルクのオン/オフを切り替えるコマンドを送信し、モーターからステータスとデータを受信します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] writeDatas モーターに書き込まれるデータを含む長整数の配列。
 * @param [in] writeDataCount パラメータ「writeDataCount」は、「writeDatas」配列内の要素の数を表す整数です。
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::MotorWRITE(byte id, long writeDatas[], int writeDataCount)
{
    byte reDummyMode = PMX::ReceiveDataOption::NoReturn;
    long dummyDatas[8];

    return this->MotorWRITE(id,writeDatas,writeDataCount,reDummyMode,dummyDatas);
}


/**
 * MotorWRITE 機能は、モータの制御やトルクのオン/オフを切り替えるコマンドを送信し、モーターからステータスとデータを受信します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] writeDatas モーターに書き込まれるデータを表す長整数の配列。
 * @param [in] writeDataCount
 * パラメータ「writeDataCount」は、「writeDatas」配列内の要素の数を表す整数です。指令値としてモーターに送信されるデータ値の数を示します。
 * @param [in] receiveMode 応答モード。返ってきたデータを変換するときに使用します。
 * @param [in] receiveData acceptData は、受信したデータが保存される長整数の配列です。サイズは 8 要素です。
 * @param [in] controlMode モータの制御モード。応答モードで現在位置を取得する時のみに使用します
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::MotorWRITE(byte id, long writeDatas[], int writeDataCount, byte receiveMode, long receiveData[8], byte controlMode)
{
    //
    //  モータに指令値を与える関数
    //

    
    
    //返すデータは初期化しておく
    if(receiveData != NULL)
    {    
        for(int i = 0;i<8;i++)
        {
            receiveData[i] = PMX::ErrorUint32Data;
        }
    }



    // int readDataSize = 0;
    // if( receiveMode != 0x00)
    // {
    //     readDataSize = this->__byteCounter(receiveMode);
    // }     

    int readDataSize = this->__byteCounter(receiveMode);

    int txSize = PMX::MinimumLength::Send + (writeDataCount * 2);   
    int rxSize = PMX::MinimumLength::Receive + 1 + readDataSize;
    
    byte txbuf[txSize]; //トルクスイッチ時は送信データ固定
    byte rxbuf[26]; //motorwriteの最大値

    
    txbuf[0] = (byte)(0xFE);           // HEADER
    txbuf[1] = (byte)(0xFE);           // HEADER
    txbuf[2] = (byte)(id);             // ID
    txbuf[3] = (byte)(txSize);        // LENGTH
    txbuf[4] = PMX::SendCmd::MotorWRITE; //Comannd
    txbuf[5] = (byte)(0x00);           // OPTION

    //送るデータをbyteに変換して代入する
    for(int i = 0 ; i < writeDataCount ; i++)
    {
        //short型、unsigned short型があるのでそれぞれ分ける
        byte wordBuff[2];
        if(writeDatas[i] < 0)
        {
            DataConv::int16ToBytes((short)writeDatas[i], wordBuff);
        }
        else
        {
            DataConv::uint16ToBytes((short)writeDatas[i], wordBuff);
        }

        txbuf[PMX::BuffPter::Data + (i*2)] = wordBuff[0];
        txbuf[PMX::BuffPter::Data + (i*2) + 1] = wordBuff[1];
    }

    //　CRCの代入
    PmxCrc16::setCrc16(txbuf);

    //　送受信をする（受信数不明版
    byte rxNowSize;
    bool rxFlag = this->synchronizeVariableRead(txbuf, txSize , rxbuf, &rxNowSize);

    //Serial.println(rxNowSize);

    this->logOutputPrint(txbuf,txSize);

    //データが来てたかどうか
    if(rxFlag == false)
    {
        return PMX::ComError::TimeOut;
    }

    //データの判定
    unsigned short errorFlag = this->checkRecv(rxbuf, PMX::SendCmd::MotorWRITE);
    if(errorFlag != PMX::ComError::OK)  //ここの部分はステータスを含まないのでOK
    {
        return errorFlag;
    }

    this->logOutputPrint(rxbuf,rxNowSize);

    //トルクスイッチ情報を代入
    // if((torqueSw != NULL) && (rxNowSize >= PMX::MinimumLength::Receive))
    // {
    //     *torqueSw = rxbuf[PMX::BuffPter::Data];
    // }

    //ステータス情報を代入
    unsigned short status = rxbuf[PMX::BuffPter::Status];

    

    //データを返さなくていい時はtrueで返す
    if(receiveMode == PMX::ReceiveDataOption::NoReturn)
    {
        return status;
    }

    // ↓↓↓↓↓↓ここからはReceiveDataの処理↓↓↓↓↓

    if(rxNowSize != rxSize) //実際受信した情報と予想していた情報が違う
    {
        status += PMX::ComError::MotorREADConvertError;  //変換エラーを含んだ情報を返す
        return  status;
    }


    byte rxMotorData[readDataSize];  //受信したデータをためておくバッファ

    //受信データを抜き出す
    for(int i = 0; i < readDataSize ; i++)
    {
        rxMotorData[i] = rxbuf[PMX::BuffPter::Data + 1 + i];
    }

    //　データの代入
    bool flag = this->__convReceiveMotorData(receiveMode, rxMotorData, readDataSize, receiveData, controlMode);
    if(flag == false)   //何かしら失敗したらエラーを付加して抜ける
    {
        status += PMX::ComError::MotorREADConvertError;  //受信エラーを含んだ情報を返す
        return  status;
    }

    //ここまでこれば合格
    return status;
}

/**
 * MotorWRITEでデータ指示一つの場合に使用します
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] targetVal 目標値(位置、速度、電流、トルク、PWM)が入ります
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::MotorWRITESingle(byte id, long targetVal)
{
    long sendData[] = {targetVal};
    return this->MotorWRITE(id, sendData, 1);
}

/**
 * MotorWRITEでデータ指示一つの場合に使用します
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] targetVal 目標値(位置、速度、電流、トルク、PWM)が入ります
 * @param [in] receiveMode モータの制御モード。応答モードで現在位置を取得する時のみに使用します。
 * @param [out] receiveData パラメータ「receiveData」は、サイズが 8 の長い整数の配列です。これは、モーターから受信したデータを保存するために使用されます。
 * @param [in] controlMode controlMode パラメータは、モーターの制御モードを指定するオプションのパラメータです。デフォルト値は 0x01 です。
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::MotorWRITESingle(byte id, long targetVal,byte receiveMode, long receiveData[8], byte controlMode)
{
    long sendData[] = {targetVal};
    return this->MotorWRITE(id, sendData, 1, receiveMode, receiveData, controlMode);
}



/**
 * MotorWriteを使用して2変数を指示するときに使用します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] targetVal1 目標値1(位置、速度、電流、トルク、PWM)が入ります
 * @param [in] targetVal2 目標値2(速度、電流、トルク、PWM、時間)が入ります
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::MotorWRITEDouble(byte id, long targetVal1, long targetVal2)
{
    long sendData[] = {targetVal1,targetVal2};
    return this->MotorWRITE(id, sendData, 2);
}

/**
 * MotorWriteを使用して2変数を指示するときに使用します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] targetVal1 目標値1(位置、速度、電流、トルク、PWM)が入ります
 * @param [in] targetVal2 目標値2(速度、電流、トルク、PWM、時間)が入ります
 * @param [in] receiveMode 応答モード。返ってきたデータを変換するときに使用します。
 * @param [out] receiveData パラメータ「receiveData」は、最大 8 つの要素を保持できる long 型の配列です。モーターから受信したデータを保存するために使用されます。
 * @param [in] controlMode モータの制御モード。応答モードで現在位置を取得する時のみに使用します。
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::MotorWRITEDouble(byte id, long targetVal1, long targetVal2,byte receiveMode, long receiveData[8], byte controlMode)
{
    long sendData[] = {targetVal1,targetVal2};
    return this->MotorWRITE(id, sendData, 2, receiveMode, receiveData, controlMode);
}

/**
 * @brief 
 * 
 * @param id 
 * @param targetVal1 
 * @param targetVal2 
 * @param targetVal3 
 * @return unsigned short 
 */
unsigned short PmxBace::MotorWRITETriple(byte id, long targetVal1, long targetVal2, long targetVal3)
{
    long sendData[] = {targetVal1,targetVal2,targetVal3};
    return this->MotorWRITE(id, sendData, 3);
}

/**
 * @brief 
 * 
 * @param id 
 * @param targetVal1 
 * @param targetVal2 
 * @param targetVal3 
 * @param receiveMode 
 * @param receiveData 
 * @param controlMode 
 * @return unsigned short 
 */
unsigned short PmxBace::MotorWRITETriple(byte id, long targetVal1, long targetVal2,long targetVal3,byte receiveMode, long receiveData[8], byte controlMode)
{
    long sendData[] = {targetVal1,targetVal2,targetVal3};
    return this->MotorWRITE(id, sendData, 3, receiveMode, receiveData, controlMode);
}




/**
 * SystemREAD 関数は、サーボからシステム情報を読み取ります。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] rxData SystemREADで取得した全てのデータをデータ配列として返します
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::SystemREAD(byte id, byte rxData[13])
{
    int readDataSize = 13;
    int txSize = PMX::MinimumLength::Send;    //memREADはLength固定
    int rxSize = PMX::MinimumLength::Receive + readDataSize;
    
    byte txbuf[txSize];
    byte rxbuf[rxSize];

    txbuf[0] = (byte)(0xFE);            // HEADER
    txbuf[1] = (byte)(0xFE);            // HEADER
    txbuf[2] = (byte)(id);              // ID
    txbuf[3] = (byte)(txSize);          // LENGTH
    txbuf[4] = PMX::SendCmd::SystemREAD;      //Comannd
    txbuf[5] = (byte)(0x00);            // OPTION

    PmxCrc16::setCrc16(txbuf);

    bool rxFlag= this->synchronize(txbuf, txSize , rxbuf, rxSize);

    this->logOutputPrint(txbuf,txSize);

    if(rxFlag == false)
    {
        //Serial.println("timeout");
        return PMX::ComError::TimeOut;
    }

    this->logOutputPrint(rxbuf,rxSize);

    unsigned short errorFlag = this->checkRecv(rxbuf, PMX::SendCmd::SystemREAD);


    if(errorFlag != PMX::ComError::OK)  //ここの部分はステータスを含まないのでOK
    {
        return errorFlag;
    }

    unsigned short status = rxbuf[PMX::BuffPter::Status];

    
    for(int i = 0; i < readDataSize ; i++)
    {
        rxData[i] = rxbuf[PMX::BuffPter::Data + i];
    }

    return status;
}


//
//SystemREADを利用したデータのget関数一覧
//


/**
 * @brief SystemREAD 関数は、サーボからシステム情報を読み取ります。
 * @details 読み取ることができるデータは下記になります。
 * @details  * PMX個体固有のシリアル番号
 * @details  * PMXの機種番号
 * @details  * バージョンの情報
 * @details  * コマンドを受け取ってから返信するまでの時間
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] serialLongNum SystemREADで取得したすべてのデータを返します。[SerialNum, ModelNum, Version, responsTime]
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getSerialNumber(byte id, unsigned long *serialLongNum)
{
    byte serialNumByte[4];
    //DataConv::uint32ToBytes(PMX::ErrorUint32Data,serialNumByte);//取得する配列は初期化しておく
    unsigned short status = this->getSerialNumber(id, serialNumByte);
    *serialLongNum = DataConv::bytesToUint32(serialNumByte);

    return status;
}


/**
 * @brief 
 * 
 * @param id 
 * @param serialByteNum 
 * @return unsigned short 
 */
unsigned short PmxBace::getSerialNumber(byte id, byte serialByteNum[4])
{
    byte sysData[13];

    unsigned short status = this->SystemREAD(id, sysData);

    //通信エラー等々はエラーとして扱う（ステータスエラー時はそのまま
    if((status & PMX::ComError::ErrorMask) != PMX::ComError::OK)
    {
        //エラーデータをbytesに変換し、代入
        DataConv::uint32ToBytes(PMX::ErrorUint32Data, serialByteNum);
        // byte errorBytesData[4]; 
        // DataConv::uint32ToBytes(PMX::ErrorUint32Data, errorBytesData);
        // for(int i = 0;i < 4;i++)    
        // {
        //     serialByteNum[i] = errorBytesData[i];
        // }
        // *serialNum = PMX::ErrorByteData;
    }
    else
    {
        for(int i = 0;i<4;i++)
        {
            serialByteNum[i] = sysData[i];
        }
    }

    return status;
}

/**
 * @brief 
 * 
 * @param id 
 * @param modelFullNum 
 * @return unsigned short 
 */
unsigned short PmxBace::getModelNum(byte id, unsigned long *modelFullNum)
{
    byte sysData[13];

    unsigned short status = this->SystemREAD(id, sysData);

    //通信エラー等々はエラーとして扱う（ステータスエラー時はそのまま
    if((status & PMX::ComError::ErrorMask) != PMX::ComError::OK)
    {
        *modelFullNum = PMX::ErrorUint32Data;
    }
    else
    {
        for(int i = 0;i<4;i++)
        {
            modelFullNum[i] = sysData[i+4];
        }
    }

    return status;
}


/**
 * @brief 
 * 
 * @param id 
 * @param modelNum 
 * @param seriesNum 
 * @return unsigned short 
 */
unsigned short PmxBace::getModelNum(byte id, unsigned short *modelNum, unsigned short *seriesNum)
{
    byte sysData[13];

    unsigned short status = this->SystemREAD(id, sysData);

    //通信エラー等々はエラーとして扱う（ステータスエラー時はそのまま
    if((status & PMX::ComError::ErrorMask) != PMX::ComError::OK)
    {
        
        *modelNum = PMX::ErrorUint16Data;
        *seriesNum = PMX::ErrorUint16Data;
    }
    else
    {
        byte modelData[2];
        modelData[0] = sysData[4]; 
        modelData[1] = sysData[5]; 

        *modelNum = DataConv::bytesToUint16(modelData);

        byte seriesData[2];
        seriesData[0] = sysData[6]; 
        seriesData[1] = sysData[7]; 

        *seriesNum = DataConv::bytesToUint16(seriesData);

    }


    return status;
}

/**
 * @brief 
 * 
 * @param id 
 * @param verData 
 * @return unsigned short 
 */
unsigned short PmxBace::getVersion(byte id,byte verData[4])
{
    byte sysData[13];

    unsigned short status = this->SystemREAD(id, sysData);

    //通信エラー等々はエラーとして扱う（ステータスエラー時はそのまま
    if((status & PMX::ComError::ErrorMask) != PMX::ComError::OK)
    {
        //*verData = PMX::ErrorByteData;
        for(int i = 0;i<4;i++)
        {
            verData[i] = PMX::ErrorByteData;
        }
    }
    else
    {
        for(int i = 0;i<4;i++)
        {
            verData[i] = sysData[i+8];
        }
    }

    return status;
}

/**
 * @brief 
 * 
 * @param id 
 * @param respTime 
 * @return unsigned short 
 */
unsigned short PmxBace::getResponseTime(byte id,byte *respTime)
{
    byte sysData[13];
    unsigned short status = this->SystemREAD(id, sysData);

//通信エラー等々はエラーとして扱う（ステータスエラー時はそのまま
    if((status & PMX::ComError::ErrorMask) != PMX::ComError::OK)
    {
        *respTime = PMX::ErrorByteData;
    }
    else
    {
        *respTime = sysData[12];
    }

    return status;
}




/**
 * @brief SystemWRITEコマンドを使用してID、ボーレート、パリティ、および応答時間を設定します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] serialNum サーボの固有番号。SystemReadでデータを返します
 * @param [in] option 
 * @param [in] newId 
 * @param [in] newBaudrateVal 
 * @param [in] newParityVal 
 * @param [in] newResponseTime 
 * @return unsigned short status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::SystemWRITE(byte id, byte serialNum[], byte option, byte newId, byte newBaudrateVal, byte newParityVal, byte newResponseTime)
{
    
    int txSize = PMX::MinimumLength::Send + 4 + 4 ;  //SystemWRITEは書き込むデータは4byte,SerialNumが4byte
    int rxSize = PMX::MinimumLength::Receive;     //SystemWRITEは固定
    
    byte txbuf[txSize];
    byte rxbuf[rxSize];

    txbuf[0] = (byte)(0xFE);            // HEADER
    txbuf[1] = (byte)(0xFE);            // HEADER
    txbuf[2] = (byte)(id);              // ID
    txbuf[3] = (byte)(txSize);          // LENGTH
    txbuf[4] = PMX::SendCmd::SystemWRITE;      //Comannd
    txbuf[5] = (byte)(option);            // OPTION

    txbuf[6] = (byte)(serialNum[0]);
    txbuf[7] = (byte)(serialNum[1]);
    txbuf[8] = (byte)(serialNum[2]);
    txbuf[9] = (byte)(serialNum[3]);
    txbuf[10] = (byte)(newId);
    txbuf[11] = (byte)(newBaudrateVal);
    txbuf[12] = (byte)(newParityVal);
    txbuf[13] = (byte)(newResponseTime);

    PmxCrc16::setCrc16(txbuf);

    bool rxFlag= this->synchronize(txbuf, txSize , rxbuf, rxSize);

    this->logOutputPrint(txbuf,txSize);

    if(rxFlag == false)
    {
        //Serial.println("timeout");
        return PMX::ComError::TimeOut;
    }

    this->logOutputPrint(rxbuf,rxSize);

    unsigned short errorFlag = this->checkRecv(rxbuf, PMX::SendCmd::SystemWRITE);


    if(errorFlag != PMX::ComError::OK)  //ここの部分はステータスを含まないのでOK
    {
        return errorFlag;
    }

    unsigned short status = rxbuf[PMX::BuffPter::Status];

    return status;
}


/**
 * @brief 
 * 
 * @param id 
 * @param serialNum 
 * @param option 
 * @param newId 
 * @param newBaudrateVal 
 * @param newParityVal 
 * @param newResponseTime 
 * @return unsigned short 
 */
unsigned short PmxBace::SystemWRITE(byte id, unsigned long serialNum, byte option, byte newId, byte newBaudrateVal, byte newParityVal, byte newResponseTime)
{
    //systemNumでLongで取得した多場合はbyte配列にする
    byte serialByteNum[4];
    DataConv::uint32ToBytes(serialNum,serialByteNum);
    //SystemWriteでデータを書き込む
    return this->SystemWRITE(id,serialByteNum,option,newId,newBaudrateVal,newParityVal,newResponseTime);
}

/**
 * @brief 
 * 
 * @param id 
 * @param option 
 * @param newId 
 * @param newBaudrateVal 
 * @param newParityVal 
 * @param newResponseTime 
 * @return unsigned short 
 */
unsigned short PmxBace::SystemWRITE(byte id, byte option, byte newId, byte newBaudrateVal, byte newParityVal, byte newResponseTime)
{
    byte serialByteNum[4];
    unsigned short serialNumGetSt = this->getSerialNumber(id,serialByteNum);
        
    //通信エラー等々はエラーとして扱う（ステータスエラー時はそのまま
    if((serialNumGetSt & PMX::ComError::ErrorMask) != PMX::ComError::OK)
    {
        return serialNumGetSt;
    }

    //SystemWriteでデータを書き込む
    return this->SystemWRITE(id,serialByteNum,option,newId,newBaudrateVal,newParityVal,newResponseTime);

}        

//
//SystemWriteを利用したデータのset関数一覧
//


/**
 * @brief 
 * 
 * @param id 
 * @param newId 
 * @return unsigned short 
 */
unsigned short PmxBace::setId(byte id, byte newId)
{
    if((newId < 0) || (newId > 239))
    {
        return PMX::ComError::FormatError;
    }
    return this->SystemWRITE(id, 0x01, newId, 0, 0, 0);
}


/**
 * @brief 
 * 
 * @param id 
 * @param newBaudRate 
 * @return unsigned short 
 */
unsigned short PmxBace::setBaudrate(byte id, byte newBaudRate)
{
    if((newBaudRate < 0) || (newBaudRate > 0x07))
    {
        return PMX::ComError::FormatError;
    }
    return this->SystemWRITE(id, 0x02, 0, newBaudRate, 0, 0);
}

/**
 * @brief 
 * 
 * @param id 
 * @param newParityNum 
 * @return unsigned short 
 */
unsigned short PmxBace::setParity(byte id, byte newParityNum)
{
    if((newParityNum < 0) || (newParityNum > 0x02))
    {
        return PMX::ComError::FormatError;
    }
    return this->SystemWRITE(id, 0x04, 0, 0, newParityNum, 0);
}

/**
 * @brief 
 * 
 * @param id 
 * @param respTime 
 * @return unsigned short 
 */
unsigned short PmxBace::setResponseTime(byte id, byte respTime)
{
    if(respTime < 1)
    {
        return PMX::ComError::FormatError;
    }
    return this->SystemWRITE(id, 0x08, 0, 0, 0, respTime);
}



/**
 * サーボモータを再起動させます。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] resetTime 再起動まで待つ時間(ms)
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::ReBoot(byte id, int resetTime)
{

    int txSize = 10;    //memREADはLength固定
    int rxSize = 8;
    
    byte txbuf[txSize];
    byte rxbuf[rxSize];

    txbuf[0] = (byte)(0xFE);            // HEADER
    txbuf[1] = (byte)(0xFE);            // HEADER
    txbuf[2] = (byte)(id);              // ID
    txbuf[3] = (byte)(txSize);          // LENGTH
    txbuf[4] = PMX::SendCmd::ReBoot;  //Comannd
    txbuf[5] = (byte)(0x00);        // OPTION
    txbuf[6] = (byte)(resetTime & 0x00ff);           //先頭アドレス（下位）
    txbuf[7] = (byte)((resetTime & 0xff00) >> 8);    //先頭アドレス（上位）

    PmxCrc16::setCrc16(txbuf);

    bool rxFlag= this->synchronize(txbuf, txSize , rxbuf, rxSize);

    this->logOutputPrint(txbuf,txSize);

    if(rxFlag == false)
    {
        //Serial.println("timeout");
        return PMX::ComError::TimeOut;
    }

    this->logOutputPrint(rxbuf,rxSize);

    unsigned short errorFlag = this->checkRecv(rxbuf, PMX::SendCmd::ReBoot);


    if(errorFlag != PMX::ComError::OK)  //ここの部分はステータスを含まないのでOK
    {
        return errorFlag;
    }

    unsigned short status = rxbuf[PMX::BuffPter::Status];

    return status;
}


/**
 * サーボを工場出荷時の設定にリセットします
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] serialNum シリアル番号
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::FactoryReset(byte id, byte serialNum[])
{

    int txSize = 12;    //memREADはLength固定
    int rxSize = 8;
    
    byte txbuf[txSize];
    byte rxbuf[rxSize];

    txbuf[0] = (byte)(0xFE);            // HEADER
    txbuf[1] = (byte)(0xFE);            // HEADER
    txbuf[2] = (byte)(id);              // ID
    txbuf[3] = (byte)(txSize);          // LENGTH
    txbuf[4] = PMX::SendCmd::FactoryReset;      //Comannd
    txbuf[5] = (byte)(0x00);            // OPTION

    txbuf[6] = (byte)(serialNum[0]);
    txbuf[7] = (byte)(serialNum[1]);
    txbuf[8] = (byte)(serialNum[2]);
    txbuf[9] = (byte)(serialNum[3]);

    PmxCrc16::setCrc16(txbuf);

    bool rxFlag= this->synchronize(txbuf, txSize , rxbuf, rxSize);

    this->logOutputPrint(txbuf,txSize);

    if(rxFlag == false)
    {
        //Serial.println("timeout");
        return PMX::ComError::TimeOut;
    }

    this->logOutputPrint(rxbuf,rxSize);

    unsigned short errorFlag = this->checkRecv(rxbuf, PMX::SendCmd::FactoryReset);


    if(errorFlag != PMX::ComError::OK)  //ここの部分はステータスを含まないのでOK
    {
        return errorFlag;
    }

    unsigned short status = rxbuf[PMX::BuffPter::Status];

    return status;
}


//
//MemREADを利用したデータのget関数一覧
//

/**
 * @brief 
 * 
 * @param id 
 * @param data 
 * @return unsigned short 
 */
unsigned short PmxBace::getPositionKpGain(byte id, unsigned long *data)
{
    return this->MemREADToUint32(id, PMX::RamAddrList::PositionKp, data);
}

/**
 * @brief 
 * 
 * @param id 
 * @param data 
 * @return unsigned short 
 */
unsigned short PmxBace::getPositionKiGain(byte id, unsigned long *data)
{
    return this->MemREADToUint32(id, PMX::RamAddrList::PositionKi, data);
}

/**
 * @brief 
 * 
 * @param id 
 * @param data 
 * @return unsigned short 
 */
unsigned short PmxBace::getPositionKdGain(byte id, unsigned long *data)
{
    return this->MemREADToUint32(id, PMX::RamAddrList::PositionKd, data);
}


/**
 * @brief 
 * 
 * @param id 
 * @param kpdata 
 * @param kidata 
 * @param kddata 
 * @return unsigned short 
 */
unsigned short PmxBace::getPositionGain(byte id, unsigned long *kpdata, unsigned long *kidata, unsigned long *kddata)
{
    byte rxData[12];
    unsigned short status = this->MemREAD(id, PMX::RamAddrList::PositionKp, 12, rxData);
    byte pidData[4];
    for(int i = 0; i < 4; i++)
    {
        pidData[i] = rxData[i];
    }
    *kpdata = DataConv::bytesToInt32(pidData);

    for(int i = 0; i < 4; i++)
    {
        pidData[i] = rxData[i+4];
    }
    *kidata = DataConv::bytesToInt32(pidData);

    for(int i = 0; i < 4; i++)
    {
        pidData[i] = rxData[i+8];
    }
    *kddata = DataConv::bytesToInt32(pidData);

    //通信エラーの時は0x7FFFを入れておく
    if( (status & PMX::ComError::ErrorMask) != 0x0000 )
    {
        *kpdata = (short)PMX::ErrorUint32Data;
    }

    return status;
}

/**
 * @brief 
 * 
 * @param id 
 * @param data 
 * @return unsigned short 
 */
unsigned short PmxBace::getPositionStretchGain(byte id, unsigned long *data)
{
    return this->MemREADToUint32(id, PMX::RamAddrList::PositionSt, data);
}


/**
 * @brief 
 * 
 * @param id 
 * @param data 
 * @return unsigned short 
 */
unsigned short PmxBace::getSpeedKpGain(byte id, unsigned long *data)
{
    return this->MemREADToUint32(id, PMX::RamAddrList::SpeedKp, data);
}


/**
 * @brief 
 * 
 * @param id 
 * @param data 
 * @return unsigned short 
 */
unsigned short PmxBace::getSpeedKiGain(byte id, unsigned long *data)
{
    return this->MemREADToUint32(id, PMX::RamAddrList::SpeedKi, data);
}

/**
 * @brief 
 * 
 * @param id 
 * @param data 
 * @return unsigned short 
 */
unsigned short PmxBace::getSpeedKdGain(byte id, unsigned long *data)
{
    return this->MemREADToUint32(id, PMX::RamAddrList::SpeedKd, data);
}

/**
 * @brief 
 * 
 * @param id 
 * @param kpdata 
 * @param kidata 
 * @param kddata 
 * @return unsigned short 
 */
unsigned short PmxBace::getSpeedGain(byte id, unsigned long *kpdata, unsigned long *kidata, unsigned long *kddata)
{
    byte rxData[12];
    unsigned short status = this->MemREAD(id, PMX::RamAddrList::SpeedKp, 12, rxData);
    byte pidData[4];
    for(int i = 0; i < 4; i++)
    {
        pidData[i] = rxData[i];
    }
    *kpdata = DataConv::bytesToInt32(pidData);

    for(int i = 0; i < 4; i++)
    {
        pidData[i] = rxData[i+4];
    }
    *kidata = DataConv::bytesToInt32(pidData);

    for(int i = 0; i < 4; i++)
    {
        pidData[i] = rxData[i+8];
    }
    *kddata = DataConv::bytesToInt32(pidData);

    //通信エラーの時は0x7FFFを入れておく
    if( (status & PMX::ComError::ErrorMask) != 0x0000 )
    {
        *kpdata = (short)PMX::ErrorUint32Data;
    }

    return status;
}

/**
 * 関数 getCurrentKpGain は、特定の ID の現在の比例ゲイン値を取得し、それをデータ パラメーターに保存します。
 * 
 * @param id 「id」パラメータは、Kp ゲインが取得されるデバイスまたはコンポーネントの ID を表すバイトです。
 * @param data
 * 「data」パラメータは、符号なしのlong変数へのポインタです。この変数は、「id」パラメータと「PMX::RamAddrList::CurrentKp」定数で指定されたメモリ
 * アドレスから読み取られた値を格納するために使用されます。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::getCurrentKpGain(byte id, unsigned long *data)
{
    return this->MemREADToUint32(id, PMX::RamAddrList::CurrentKp, data);
}

/**
 * @brief 
 * 
 * @param id 
 * @param data 
 * @return unsigned short 
 */
unsigned short PmxBace::getCurrentKiGain(byte id, unsigned long *data)
{
    return this->MemREADToUint32(id, PMX::RamAddrList::CurrentKi, data);
}

/**
 * @brief 
 * 
 * @param id 
 * @param data 
 * @return unsigned short 
 */
unsigned short PmxBace::getCurrentKdGain(byte id, unsigned long *data)
{
    return this->MemREADToUint32(id, PMX::RamAddrList::CurrentKd, data);
}

/**
 * @brief 
 * 
 * @param id 
 * @param kpdata 
 * @param kidata 
 * @param kddata 
 * @return unsigned short 
 */
unsigned short PmxBace::getCurrentGain(byte id, unsigned long *kpdata, unsigned long *kidata, unsigned long *kddata)
{
    byte rxData[12];
    unsigned short status = this->MemREAD(id, PMX::RamAddrList::CurrentKp, 12, rxData);
    byte pidData[4];
    for(int i = 0; i < 4; i++)
    {
        pidData[i] = rxData[i];
    }
    *kpdata = DataConv::bytesToInt32(pidData);

    for(int i = 0; i < 4; i++)
    {
        pidData[i] = rxData[i+4];
    }
    *kidata = DataConv::bytesToInt32(pidData);

    for(int i = 0; i < 4; i++)
    {
        pidData[i] = rxData[i+8];
    }
    *kddata = DataConv::bytesToInt32(pidData);

    //通信エラーの時は0x7FFFを入れておく
    if( (status & PMX::ComError::ErrorMask) != 0x0000 )
    {
        *kpdata = (short)PMX::ErrorUint32Data;
    }

    return status;
}

/**
 * 関数「getTorqueKpGain」は、特定の ID のトルク Kp ゲインを返します。
 * 
 * @param id 「id」パラメータは、トルク Kp ゲインが取得されるデバイスまたはコンポーネントの ID を表すバイトです。
 * @param data 「data」パラメータは、符号なしのlong変数へのポインタです。この変数は、「id」および「PMX::RamAddrList::TorqueKp」パラメータで指定されたメモリ
 * アドレスから読み取られた値を格納するために使用されます。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::getTorqueKpGain(byte id, unsigned long *data)
{
    return this->MemREADToUint32(id, PMX::RamAddrList::TorqueKp, data);
}

/**
 * 関数「getTorqueKiGain」は、特定の ID のトルク Ki ゲインを返します。
 * 
 * @param id 「id」パラメータは、トルク Ki ゲインが取得されるデバイスの ID を表すバイトです。
 * @param data 「data」パラメータは、符号なしのlong変数へのポインタです。この変数は、「id」および「PMX::RamAddrList::TorqueKi」パラメータで指定されたメモリ
 * アドレスから読み取られた値を格納するために使用されます。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::getTorqueKiGain(byte id, unsigned long *data)
{
    return this->MemREADToUint32(id, PMX::RamAddrList::TorqueKi, data);
}

/**
 * 関数「getTorqueKdGain」は、特定の ID のトルク Kd ゲインを返します。
 * 
 * @param id 「id」パラメータは、トルク Kd ゲインが取得されるデバイスの ID を表すバイトです。
 * @param data 「data」パラメータは、符号なしのlong変数へのポインタです。この変数は、「id」および「PMX::RamAddrList::TorqueKd」パラメータで指定されたメモリ
 * アドレスから読み取られた値を格納するために使用されます。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::getTorqueKdGain(byte id, unsigned long *data)
{
    return this->MemREADToUint32(id, PMX::RamAddrList::TorqueKd, data);
}

/**
 * 関数 `getTorqueGain` は、特定のモーター ID のトルク ゲイン値 (kp、ki、kd) を取得し、提供された変数に格納し、通信のステータスを返します。
 * 
 * @param id パラメータ「id」は「byte」タイプで、トルク ゲイン値が取得されるデバイスの ID を表します。
 * @param kpdata トルク ゲイン Kp の値が格納される unsigned long 変数へのポインタ。
 * @param kidata `kidata` パラメータは、unsigned long 変数へのポインタです。トルク制御の積分ゲインの値を記憶するために使用します。
 * @param kddata パラメータ `kddata` は、unsigned long 変数へのポインタです。デバイスとの通信から得られる微分ゲイン（Kd）の値を保存するために使用されます。
 * 
 * @return unsigned short 型の変数。
 */
unsigned short PmxBace::getTorqueGain(byte id, unsigned long *kpdata, unsigned long *kidata, unsigned long *kddata)
{
    byte rxData[12];
    unsigned short status = this->MemREAD(id, PMX::RamAddrList::TorqueKp, 12, rxData);
    byte pidData[4];
    for(int i = 0; i < 4; i++)
    {
        pidData[i] = rxData[i];
    }
    *kpdata = DataConv::bytesToInt32(pidData);

    for(int i = 0; i < 4; i++)
    {
        pidData[i] = rxData[i+4];
    }
    *kidata = DataConv::bytesToInt32(pidData);

    for(int i = 0; i < 4; i++)
    {
        pidData[i] = rxData[i+8];
    }
    *kddata = DataConv::bytesToInt32(pidData);

    //通信エラーの時は0x7FFFを入れておく
    if( (status & PMX::ComError::ErrorMask) != 0x0000 )
    {
        *kpdata = (short)PMX::ErrorUint32Data;
    }

    return status;
}

/**
 * @brief 指定した ID の中央値オフセット値を取得します。
 * @details MemREADを使用して指定したIDから中央値オフセット値を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] offsetData 中央値オフセット値
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getCenterOffset(byte id, short *offsetData)
{
    return this->MemREADToInt16(id, PMX::RamAddrList::CenterOffset, offsetData);
}

/**
 * @brief 指定した ID のクローン/リバース値を取得します。
 * @details MemREADを使用して指定したIDからクローン/リバースを取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] data クローン/リバース値
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getCloneReverse(byte id, byte *data)
{
    return this->MemREADToByte(id, PMX::RamAddrList::CloneReverse, data);
}

/**
 * @brief 指定したIDの下限電圧リミット値を取得します。
 * @details MemREADを使用して指定したIDから下限電圧リミット値を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] data 下限電圧設定値
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getMinVoltageLimit(byte id, unsigned short *data)
{
    return this->MemREADToUint16(id, PMX::RamAddrList::MinVoltageLimit, data);
}

/**
 * @brief 指定したIDの下限リミット電圧を下回った場合の出力割合を取得します。
 * @details MemREADを使用して指定したIDから下限リミット電圧を下回った場合の出力割合を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] data 下限リミット電圧を下回った時の出力割合(%)
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getMinVoltageLimitPower(byte id, unsigned short *data)
{
    return this->MemREADToUint16(id, PMX::RamAddrList::MinVoltageLimitPower, data);
}

/**
 * @brief 指定したIDの上限リミット電圧を取得します。
 * @details MemREADを使用して指定したIDから上限リミット電圧を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] data 入力電圧最大値
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getMaxVoltageLimit(byte id, unsigned short *data)
{
    return this->MemREADToUint16(id, PMX::RamAddrList::MaxVoltageLimit, data);
}

/**
 * @brief 指定したIDの上限リミット電圧を超えた場合の出力割合を取得します。
 * @details MemREADを使用して指定したIDから上限リミット電圧を超えた場合の出力割合を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] data 上限リミット電圧を上回った時の出力割合(%)
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getMaxVoltageLimitPower(byte id, unsigned short *data)
{
    return this->MemREADToUint16(id, PMX::RamAddrList::MaxVoltageLimitPower, data);
}

/**
 * @brief 指定したIDの電流リミット値を取得します。
 * @details MemREADを使用して指定したIDから電流リミット値を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] data 電流リミット値
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getCurrentLimit(byte id, unsigned short *data)
{
    return this->MemREADToUint16(id, PMX::RamAddrList::CurrentLimit, data);
}

/**
 * @brief 指定したIDの電流リミットを超えた時の出力割合を取得します。
 * @details MemREADを使用して指定したIDから電流リミットを超えた時の出力割合を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] data 電流リミット時の出力割合(%)
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getCurrentLimitPower(byte id, unsigned short *data)
{
    return this->MemREADToUint16(id, PMX::RamAddrList::CurrentLimitPower, data);
}

/**
 * @brief 指定したIDのモータ温度リミット値を取得します。
 * @details MemREADを使用して指定したIDからモータ温度リミット値を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] data モータ温度リミット値
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getMotorTempLimit(byte id, unsigned short *data)
{
    return this->MemREADToUint16(id, PMX::RamAddrList::MotorTempLimit, data);
}

/**
 * @brief 指定したIDのモータ温度リミットを超えた時の出力割合を取得します。
 * @details MemREADを使用して指定したIDからモータ温度リミットを超えた時の出力割合を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] data モータ温度リミット時の出力割合(%)
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)。
 */
unsigned short PmxBace::getMotorTempLimitPower(byte id, unsigned short *data)
{
    return this->MemREADToUint16(id, PMX::RamAddrList::MotorTempLimitPower, data);
}

/**
 * @brief 指定したIDのCPU温度リミット値を取得します。
 * @details MemREADを使用して指定したIDからCPU温度リミット値を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] data CPU温度リミット値
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getCpuTempLimit(byte id, unsigned short *data)
{
    return this->MemREADToUint16(id, PMX::RamAddrList::CpuTempLimit, data);
}

/**
 * @brief 指定したIDのCPU温度リミットを超えた時の出力割合を取得します。
 * @details MemREADを使用して指定したIDからCPU温度リミットを超えた時の出力割合を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] data CPU温度リミット時の出力割合(%)
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getCpuTempLimitPower(byte id, unsigned short *data)
{
    return this->MemREADToUint16(id, PMX::RamAddrList::CpuTempLimitPower, data);
}

/**
 * @brief 指定したIDのCW方向の動作角度リミット値を取得します。
 * @details MemREADを使用して指定したIDからCW方向の動作角度リミット値を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] data CW方向の動作角度リミット値
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getCwPositionLimit(byte id, short *data)
{
    return this->MemREADToInt16(id, PMX::RamAddrList::CwPositionLimit, data);
}

/**
 * @brief 指定したIDのCW側の角度を超えた時の出力の割合を取得します。
 * @details MemREADを使用して指定したIDからCW側の角度を超えた時の出力の割合を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] data CW方向の閾値外時の出力割合(%)
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getCwPositionLimitPower(byte id, unsigned short *data)
{
    return this->MemREADToUint16(id, PMX::RamAddrList::CwPositionLimitPower, data);
}

/**
 * @brief 指定したIDのCCW方向の動作角度リミット値を取得します。
 * @details MemREADを使用して指定したIDからCCW方向の動作角度リミット値を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] data CW方向の動作角度リミット値
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getCcwPositionLimit(byte id, short *data)
{
    return this->MemREADToInt16(id, PMX::RamAddrList::CcwPositionLimit, data);
}

/**
 * @brief 指定したIDのCCW側の角度を超えた時の出力の割合を取得します。
 * @details MemREADを使用して指定したIDからCCW側の角度を超えた時の出力の割合を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] data CCW方向の閾値外時の出力割合(%)
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getCcwPositionLimitPower(byte id, unsigned short *data)
{
    return this->MemREADToUint16(id, PMX::RamAddrList::CcwPositionLimitPower, data);
}

/**
 * @brief 指定したIDの指令可能な最大速度値を取得します。
 * @details MemREADを使用して指定したIDから指令可能な最大速度値を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] data 最大速度指令値
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getMaxGoalSpeed(byte id, short *data)
{
    return this->MemREADToInt16(id, PMX::RamAddrList::MaxGoalSpeed, data);
}

/**
 * @brief 指定したIDの指令可能な最大電流値を取得します。
 * @details MemREADを使用して指定したIDから指令可能な最大電流値を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] data 最大電流指令値
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getMaxGoalCurrent(byte id, short *data)
{
    return this->MemREADToInt16(id, PMX::RamAddrList::MaxGoalCurrent, data);
}

/**
 * @brief 指定したIDの指令可能な最大推定トルク値を取得します。
 * @details MemREADを使用して指定したIDから指令可能な最大推定トルク値を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] data 最大動推定トルク指令値
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getMaxGoalTorque(byte id, short *data)
{
    return this->MemREADToInt16(id, PMX::RamAddrList::MaxGoalTorque, data);
}

/**
 * @brief 指定したIDのモータに出力する割合を取得します。
 * @details MemREADを使用して指定したIDからモータに出力する割合を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] data モータ出力割合(%)
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getTotalPowerRate(byte id, unsigned short *data)
{
    return this->MemREADToUint16(id, PMX::RamAddrList::TotalPowerRate, data);
}

/**
 * @brief 指定したIDのロック検知を認識するまでの時間を取得します。
 * @details MemREADを使用して指定したIDからロック検知を認識するまでの時間を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] data ロック検知を認識するまでの時間
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getLockDetectTime(byte id, unsigned short *data)
{
    return this->MemREADToUint16(id, PMX::RamAddrList::LockDetectTime, data);
}

/**
 * @brief 指定したIDのロック検知を開始する出力指示の閾値を取得します。
 * @details MemREADを使用して指定したIDからロック検知を開始する出力指示の閾値を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] data ロック検知開始出力(%)
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getLockThresholdPower(byte id, unsigned short *data)
{
    return this->MemREADToUint16(id, PMX::RamAddrList::LockThresholdPower, data);
}

/**
 * @brief 指定したIDがロック検知が起きた時の出力の割合を取得します。
 * @details MemREADを使用して指定したIDからロック検知が起きた時の出力の割合を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] data ロック検知時の出力割合(%)
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getLockDetectOutputPower(byte id, unsigned short *data)
{
    return this->MemREADToUint16(id, PMX::RamAddrList::LockDetectOutputPower, data);
}

/**
 * @brief 指定したIDの現在指示位置を取得します。
 * @details 指定したIDの位置制御を含む時の現在位置を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] posData 位置制御を含む時の現在位置
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getPosition(byte id, short *posData)
{
    return this->MemREADToInt16(id, PMX::RamAddrList::NowPosition, posData);
}

/**
 * @brief 指定したIDの現在指示位置を取得します。
 * @details 指定したIDの位置制御を含まない時の現在位置を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] posData 位置制御を含まない現在位置
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getPosition(byte id, unsigned short *posData)
{
    return this->MemREADToUint16(id, PMX::RamAddrList::NowPosition, posData);
}


/**
 * @brief 指定したIDの現在指示位置を取得します。
 * @details 指定したIDの現在位置を取得します。 位置モード、それ以外で変換方法が異なります。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] posData 現在位置が代入されます
 * @param [in] controlMode 制御モード。モードにより変換が変わり、デフォルトは位置モードです。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::getPosition(byte id, long *posData, byte controlMode)
{
    unsigned short statas;
    if((controlMode & PMX::ControlMode::Position) != 0)
    {
        short bufData;
        statas = this->MemREADToInt16(id, PMX::RamAddrList::NowPosition, &bufData);
        *posData = (long)bufData;

    }
    else
    {
        unsigned short bufData;
        statas = this->MemREADToUint16(id, PMX::RamAddrList::NowPosition, &bufData);
        *posData = (long)bufData;
    }

    return statas;
}



/**
 * @brief　指定した ID の現在の速度を取得します。
 * @details　MemREADを使用して指定したIDから現在速度を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] spdData 現在の速度データ
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getSpeed(byte id, short *spdData)
{
    return this->MemREADToInt16(id, PMX::RamAddrList::NowSpeed, spdData);
}

/**
 * @brief 指定した ID の現在の電流を取得します。
 * @details　MemREADを使用して指定したIDから現在電流を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] curData 現在の電流データ
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getCurrent(byte id, short *curData)
{
    return this->MemREADToInt16(id, PMX::RamAddrList::NowCurrent, curData);
}

/**
 * @brief 指定した ID の現在のトルクを取得します。
 * @details MemREADを使用して指定したIDから現在の推定トルクを取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] trqData 現在の推定トルクデータ
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getTorque(byte id, short *trqData)
{
    return this->MemREADToInt16(id, PMX::RamAddrList::NowTorque, trqData);
}

/**
 * @brief 指定したIDの現在PWMを取得します。
 * @details MemREADを使用して指定したIDから現在PWMを取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] pwmData 現在のPWMデータ
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getPwm(byte id, short *pwmData)
{
    return this->MemREADToInt16(id, PMX::RamAddrList::NowPwm, pwmData);
}

/**
 * @brief 指定したIDの現在のモータ温度を取得します。
 * @details MemREADを使用して指定したIDから現在モータ温度を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] motTempData 現在のモータ温度データ
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getMotorTemp(byte id, short *motTempData)
{
    return this->MemREADToInt16(id, PMX::RamAddrList::MotorTemp, motTempData);
}

/**
 * @brief 指定された ID の現在のCPU温度を取得します。
 * @details MemREADを使用して指定したIDから現在CPU温度を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] cpuTempData 現在のCPU温度データ
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getCPUTemp(byte id, short *cpuTempData)
{
    return this->MemREADToInt16(id, PMX::RamAddrList::CPUTemp, cpuTempData);
}

/**
 * @brief 指定された ID の現在の電圧を取得します。
 * @details MemREADを使用して指定したIDから現在電圧を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] volData 現在の電圧データ
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getInputVoltage(byte id, unsigned short *volData)
{
    return this->MemREADToUint16(id, PMX::RamAddrList::InputVoltage, volData);
}

/**
 * @brief 指定された ID の現在の補間時間を取得します。
 * @details MemREADを使用して指定したIDから現在補間時間を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] traTimeData 現在の補間時間データ
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getTrajectoryTime(byte id, unsigned short *traTimeData)
{
    return this->MemREADToUint16(id, PMX::RamAddrList::TrajectoryTime, traTimeData);
}

/**
 * @brief 指定された ID のエンコーダを取得します。
 * @details MemREADを使用して指定したIDからエンコーダを取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] encData エンコーダデータ
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getEncoder(byte id, unsigned short *encData)
{
    return this->MemREADToUint16(id, PMX::RamAddrList::EncoderValue, encData);
}

/**
 * @brief 指定したIDのステータスを取得します。
 * @details MemREADを使用して指定したIDからステータスを取得します。
 * @details 詳しくは取扱説明書をご覧ください。<A HREF= https://kondo-robot.com/faq/pmx-servo-series-online-manual>PMXのオンラインマニュアル</A>
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] stData ステータスデータ
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getStatus(byte id, byte *stData)
{
    return this->MemREADToByte(id, PMX::RamAddrList::ErrorStatus, stData);
}

/**
 * @brief 指定された ID のシステムステータスを取得します。
 * @details MemREADを使用して指定したIDからシステムステータスを取得します。
 * @details 詳しくは取扱説明書をご覧ください。<A HREF= https://kondo-robot.com/faq/pmx-servo-series-online-manual>PMXのオンラインマニュアル</A>
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] sysStaData システムステータスデータ
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getSystemStatus(byte id, byte *sysStaData)
{
    return this->MemREADToByte(id, PMX::RamAddrList::ErrorSystem, sysStaData);
}

/**
 * @brief 指定したIDのモータステータスを取得します。
 * @details MemREADを使用して指定したIDからモータステータスを取得します。
 * @details 詳しくは取扱説明書をご覧ください。<A HREF= https://kondo-robot.com/faq/pmx-servo-series-online-manual>PMXのオンラインマニュアル</A>
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] motStaData モータステータスデータ
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getMotorStatus(byte id, byte *motStaData)
{
    return this->MemREADToByte(id, PMX::RamAddrList::ErrorMotor, motStaData);
}

/**
 * @brief 指定された ID のRAMアクセスステータスを取得します。
 * @details MemREADを使用して指定したIDからRAMアクセスステータスを取得します。
 * @details 詳しくは取扱説明書をご覧ください。<A HREF= https://kondo-robot.com/faq/pmx-servo-series-online-manual>PMXのオンラインマニュアル</A>
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] ramStaData アクセスエラーになったRAMの先頭アドレス
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getRamAccessStatus(byte id, unsigned short *ramStaData)
{
    return this->MemREADToUint16(id, PMX::RamAddrList::ErrorRamAccess, ramStaData);
}


/**
 * 関数「getFullStatus」は、PMX デバイスの完全なステータスを取得し、提供された変数に保存します。
 * 
 * @param id 「id」パラメータは、ステータスを取得するデバイスまたはコンポーネントの ID を表すバイトです。
 * @param sysSt sysSt パラメータは、dummyRead 配列から取得したシステム ステータス値を格納するバイト変数へのポインタです。
 * @param motorSt 「motorSt」パラメータはバイト変数へのポインタです。 「dummyRead」配列から取得したモーターステータス値を保存するために使用されます。
 * @param ramSt ramSt は、unsigned short 変数へのポインタです。これは、dummyRead 配列から取得された RAM ステータスの値を保存するために使用されます。
 * 
 * @return unsigned short 型の変数。
 */
unsigned short PmxBace::getFullStatus(byte id, byte *sysSt, byte *motorSt, unsigned short *ramSt)
{
    byte dummyRead[6];
    
    unsigned short status = this->MemREAD(id, PMX::RamAddrList::ErrorStatus, 6, dummyRead); 

    *sysSt = dummyRead[1];
    *motorSt = dummyRead[2];
    *ramSt = DataConv::bytesToUint16(&(dummyRead[4]));

    return status;

}


/**
 * @brief 指定された ID の全てのステータス情報をリセットします。
 * @details MemREADを使用して指定したIDから全てのステータス情報を取得します。
 * @details 詳しくは取扱説明書をご覧ください。<A HREF= https://kondo-robot.com/faq/pmx-servo-series-online-manual>PMXのオンラインマニュアル</A>
 * 
 * @param [in] id PMXサーボモータのID番号
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::resetFullStatus(byte id)
{
    byte dammyRead[6];
    return this->MemREAD(id, PMX::RamAddrList::ErrorStatus, 6, dammyRead);
}


/**
 * @brief 指定した ID のトルク ON 値を取得します。
 * @details MemREADを使用して指定したIDからトルクON値を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] trqSwitchData FreeやTorqueOnの値(PMX::TorqueSwitchTypeを参照)
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getTorqueSwitch(byte id, byte *trqSwitchData)
{
    return this->MemREADToByte(id, PMX::RamAddrList::TorqueSwitch, trqSwitchData);
}

/**
 * @brief 指定した ID の制御モードを取得します。
 * @details MemREADを使用して指定したIDから制御モードを取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] controlMode 制御モード値。PMX::ControlModeを参照してください。
        
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getControlMode(byte id, byte *controlMode)
{
    return this->MemREADToByte(id, PMX::RamAddrList::ControlMode, controlMode); 
}

/**
 * @brief 指定した ID の応答データ指定値を取得します。
 * @details MemREADを使用して指定したIDから応答データ指定値を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] receiveMode 応答データ指定値(PMX::ReceiveDataOptionを参照)
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getMotorReceive(byte id, byte *receiveMode)
{
    return this->MemREADToByte(id, PMX::RamAddrList::MotorReceiveData, receiveMode);
}

/**
 * @brief 指定した ID の補間制御軌道生成タイプ指定を取得します。
 * @details MemREADを使用して指定したIDから補間制御軌道生成タイプ指定を取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [out] traData 補間制御軌道生成タイプ指定。PMX::TrajectoryTypeを参照してください。
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::getTrajectory(byte id, byte *traData)
{
    return this->MemREADToByte(id, PMX::RamAddrList::Trajectory, traData);
}



/**
 * 関数「getCenterOffsetRange」は、指定された ID の最小および最大の中心オフセット値を取得し、通信ステータスを返します。
 * 
 * @param id 「id」パラメータは、中心オフセット範囲が取得されるデバイスまたはコンポーネントの識別子です。
 * @param minData 中心オフセット範囲の最小値を格納する short 型の変数へのポインタ。
 * @param maxData 「maxData」パラメータは、中心オフセット範囲の最大値を格納する short 変数へのポインタです。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::getCenterOffsetRange(byte id, short *minData, short *maxData)
{
    byte rxData[4];
    unsigned short status = this->MemREAD(id, PMX::RamAddrList::CenterOffsetMinRange, 4, rxData);

    byte rangeData[2];

    rangeData[0] = rxData[0];
    rangeData[1] = rxData[1];
    *minData = DataConv::bytesToInt16(rangeData);

    rangeData[0] = rxData[2];
    rangeData[1] = rxData[3];
    *maxData = DataConv::bytesToInt16(rangeData);

    //通信エラーの時は0x7FFFを入れておく
    if( (status & PMX::ComError::ErrorMask) != 0x0000 )
    {
        *minData = (short)PMX::ErrorUint16Data;
    }

    return status;
}



/**
 * この関数は、特定の ID の最小および最大電圧制限範囲を取得し、通信ステータスを返します。
 * 
 * @param id 「id」パラメータは、最小電圧制限範囲を取得するデバイスまたはコンポーネントの識別子です。
 * @param minData 最小電圧制限範囲が格納される unsigned short 変数へのポインター。
 * @param maxData 「maxData」パラメータは、符号なし short 変数へのポインタです。デバイスから取得した最大電圧制限範囲値を保存するために使用されます。
 * 
 * @return unsigned short 型の変数。
 */
unsigned short PmxBace::getMinVoltageLimitRange(byte id, unsigned short *minData, unsigned short *maxData)
{
    byte rxData[4];
    unsigned short status = this->MemREAD(id, PMX::RamAddrList::MinVoltageMinRange, 4, rxData);

    byte rangeData[2];

    rangeData[0] = rxData[0];
    rangeData[1] = rxData[1];
    *minData = DataConv::bytesToUint16(rangeData);

    rangeData[0] = rxData[2];
    rangeData[1] = rxData[3];
    *maxData = DataConv::bytesToUint16(rangeData);

    //通信エラーの時は0x7FFFを入れておく
    if( (status & PMX::ComError::ErrorMask) != 0x0000 )
    {
        *minData = (short)PMX::ErrorUint16Data;
    }

    return status;
}


/**
 * 関数「getMaxVoltageLimitRange」は、指定された ID の最小および最大電圧制限範囲を取得し、操作のステータスを返します。
 * 
 * @param id 「id」パラメータは、最大電圧制限範囲を取得するデバイスの識別子です。
 * @param minData 最小電圧制限範囲が格納される unsigned short 変数へのポインター。
 * @param maxData 「maxData」パラメータは、符号なし short 変数へのポインタです。これは、「rxData」配列から取得した最大電圧制限範囲値を保存するために使用されます。
 * 
 * @return unsigned short 型の変数。
 */
unsigned short PmxBace::getMaxVoltageLimitRange(byte id, unsigned short *minData, unsigned short *maxData)
{
    byte rxData[4];
    unsigned short status = this->MemREAD(id, PMX::RamAddrList::MaxVoltageMinRange, 4, rxData);

    byte rangeData[2];

    rangeData[0] = rxData[0];
    rangeData[1] = rxData[1];
    *minData = DataConv::bytesToUint16(rangeData);

    rangeData[0] = rxData[2];
    rangeData[1] = rxData[3];
    *maxData = DataConv::bytesToUint16(rangeData);

    //通信エラーの時は0x7FFFを入れておく
    if( (status & PMX::ComError::ErrorMask) != 0x0000 )
    {
        *minData = (short)PMX::ErrorUint16Data;
    }

    return status;
}

/**
 * 関数 `getCurrentLimitRange` は、特定の ID の電流制限範囲を取得し、最小値と最大値をそれぞれ `minData` 変数と `maxData` 変数に保存します。
 * 
 * @param id 「id」パラメータは、電流制限範囲を取得するデバイスの識別子です。
 * @param minData 最小データ値が格納される unsigned short 変数へのポインター。
 * @param maxData 「maxData」パラメータは、符号なし short 変数へのポインタです。これは、「rxData」配列から取得した最大電流制限範囲値を保存するために使用されます。
 * 
 * @return unsigned short 型の変数。
 */
unsigned short PmxBace::getCurrentLimitRange(byte id, unsigned short *minData, unsigned short *maxData)
{
    byte rxData[4];
    unsigned short status = this->MemREAD(id, PMX::RamAddrList::CurrentMinRange, 4, rxData);

    byte rangeData[2];

    rangeData[0] = rxData[0];
    rangeData[1] = rxData[1];
    *minData = DataConv::bytesToUint16(rangeData);

    rangeData[0] = rxData[2];
    rangeData[1] = rxData[3];
    *maxData = DataConv::bytesToUint16(rangeData);

    //通信エラーの時は0x7FFFを入れておく
    if( (status & PMX::ComError::ErrorMask) != 0x0000 )
    {
        *minData = (short)PMX::ErrorUint16Data;
    }

    return status;
}


/**
 * 関数「getMotorTempLimitRange」は、指定されたモーター ID の最小および最大モーター温度制限範囲を取得します。
 * 
 * @param id 「id」パラメータは、モーター温度制限範囲を取得するモーターの識別子です。
 * @param minData 「minData」パラメータは、最小モータ温度制限範囲が保存される「short」変数へのポインタです。
 * @param maxData 「maxData」パラメータは、最大モータ温度制限範囲値を格納する short 変数へのポインタです。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::getMotorTempLimitRange(byte id, short *minData, short *maxData)
{
    byte rxData[4];
    unsigned short status = this->MemREAD(id, PMX::RamAddrList::MotorTempMinRange, 4, rxData);

    byte rangeData[2];

    rangeData[0] = rxData[0];
    rangeData[1] = rxData[1];
    *minData = DataConv::bytesToInt16(rangeData);

    rangeData[0] = rxData[2];
    rangeData[1] = rxData[3];
    *maxData = DataConv::bytesToInt16(rangeData);

    //通信エラーの時は0x7FFFを入れておく
    if( (status & PMX::ComError::ErrorMask) != 0x0000 )
    {
        *minData = (short)PMX::ErrorUint16Data;
    }

    return status;
}



/**
 * 関数 getCpuTempLimitRange は、指定された ID の最小および最大 CPU 温度制限を取得し、通信ステータスを返します。
 * 
 * @param id 「id」パラメータは、CPU 温度制限範囲を取得するデバイスまたはコンポーネントの識別子です。
 * @param minData 最小 CPU 温度制限範囲が格納される変数へのポインター。
 * @param maxData CPU 温度の最大制限範囲が格納される変数へのポインター。
 * 
 * @return 「unsigned short」タイプの変数「status」の値。
 */
unsigned short PmxBace::getCpuTempLimitRange(byte id, short *minData, short *maxData)
{
    byte rxData[4];
    unsigned short status = this->MemREAD(id, PMX::RamAddrList::CpuTempMinRange, 4, rxData);

    byte rangeData[2];

    rangeData[0] = rxData[0];
    rangeData[1] = rxData[1];
    *minData = DataConv::bytesToInt16(rangeData);

    rangeData[0] = rxData[2];
    rangeData[1] = rxData[3];
    *maxData = DataConv::bytesToInt16(rangeData);

    //通信エラーの時は0x7FFFを入れておく
    if( (status & PMX::ComError::ErrorMask) != 0x0000 )
    {
        *minData = (short)PMX::ErrorUint16Data;
    }

    return status;
}




/**
 * getCwPositionLimitRange 関数は、指定された ID の最小および最大位置制限範囲を取得し、通信ステータスを返します。
 * 
 * @param id 「id」パラメータは、CW 位置制限範囲を取得するデバイスの ID です。
 * @param minData デバイスから取得した最小データ値を格納する short 型の変数へのポインター。
 * @param maxData 「maxData」パラメータは、通信から取得した最大データ値を格納する短い変数へのポインタです。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::getCwPositionLimitRange(byte id, short *minData, short *maxData)
{
    byte rxData[4];
    unsigned short status = this->MemREAD(id, PMX::RamAddrList::CwPositionMinRange, 4, rxData);

    byte rangeData[2];

    rangeData[0] = rxData[0];
    rangeData[1] = rxData[1];
    *minData = DataConv::bytesToInt16(rangeData);

    rangeData[0] = rxData[2];
    rangeData[1] = rxData[3];
    *maxData = DataConv::bytesToInt16(rangeData);

    //通信エラーの時は0x7FFFを入れておく
    if( (status & PMX::ComError::ErrorMask) != 0x0000 )
    {
        *minData = (short)PMX::ErrorUint16Data;
    }

    return status;
}

/**
 * getCcwPositionLimitRange 関数は、指定された ID の最小および最大位置制限範囲を取得し、通信ステータスを返します。
 * 
 * @param id 「id」パラメータは、位置制限範囲を取得するデバイスの ID です。
 * @param minData 最小位置制限範囲値が格納される変数へのポインタ。
 * @param maxData 「maxData」パラメータは、CCW（反時計回り）位置制限範囲の最大値を格納する「short」型の変数へのポインタです。
 * 
 * @return 「unsigned short」タイプの変数「status」の値。
 */

unsigned short PmxBace::getCcwPositionLimitRange(byte id, short *minData, short *maxData)
{
    byte rxData[4];
    unsigned short status = this->MemREAD(id, PMX::RamAddrList::CcwPositionMinRange, 4, rxData);

    byte rangeData[2];

    rangeData[0] = rxData[0];
    rangeData[1] = rxData[1];
    *minData = DataConv::bytesToInt16(rangeData);

    rangeData[0] = rxData[2];
    rangeData[1] = rxData[3];
    *maxData = DataConv::bytesToInt16(rangeData);

    //通信エラーの時は0x7FFFを入れておく
    if( (status & PMX::ComError::ErrorMask) != 0x0000 )
    {
        *minData = (short)PMX::ErrorUint16Data;
    }

    return status;
}

/**
 * 関数 `getMaxGoalSpeedRange` は、指定された ID の最小および最大の目標速度範囲を取得し、通信ステータスを返します。
 * 
 * @param id 「id」パラメータは、最大目標速度範囲を取得するデバイスの ID です。
 * @param minData 目標速度範囲の最小値を格納する short 型の変数へのポインター。
 * @param maxData 「maxData」パラメータは、最大目標速度範囲値を格納する「short」変数へのポインタです。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::getMaxGoalSpeedRange(byte id, short *minData, short *maxData)
{
    byte rxData[4];
    unsigned short status = this->MemREAD(id, PMX::RamAddrList::MaxGoalSpeedMinRange, 4, rxData);

    byte rangeData[2];

    rangeData[0] = rxData[0];
    rangeData[1] = rxData[1];
    *minData = DataConv::bytesToInt16(rangeData);

    rangeData[0] = rxData[2];
    rangeData[1] = rxData[3];
    *maxData = DataConv::bytesToInt16(rangeData);

    //通信エラーの時は0x7FFFを入れておく
    if( (status & PMX::ComError::ErrorMask) != 0x0000 )
    {
        *minData = (short)PMX::ErrorUint16Data;
    }

    return status;
}


/**
 * この関数は、特定の ID の最大目標電流範囲を取得し、指定された変数に最小値と最大値を保存します。
 * 
 * @param id 「id」パラメータは、最大目標電流範囲を取得するデバイスまたはコンポーネントの識別子です。
 * @param minData 最小目標電流範囲が格納される short 変数へのポインター。
 * @param maxData 「maxData」パラメータは、最大目標電流範囲値を格納する short 変数へのポインタです。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::getMaxGoalCurrentRange(byte id, short *minData, short *maxData)
{
    byte rxData[4];
    unsigned short status = this->MemREAD(id, PMX::RamAddrList::MaxGoalCurrentMinRange, 4, rxData);

    byte rangeData[2];

    rangeData[0] = rxData[0];
    rangeData[1] = rxData[1];
    *minData = DataConv::bytesToInt16(rangeData);

    rangeData[0] = rxData[2];
    rangeData[1] = rxData[3];
    *maxData = DataConv::bytesToInt16(rangeData);

    //通信エラーの時は0x7FFFを入れておく
    if( (status & PMX::ComError::ErrorMask) != 0x0000 )
    {
        *minData = (short)PMX::ErrorUint16Data;
    }

    return status;
}



/**
 * この関数は、指定された ID の最小および最大目標トルク範囲を取得し、通信ステータスを返します。
 * 
 * @param id 「id」パラメータは、最大目標トルク範囲を取得するデバイスの ID です。
 * @param minData 「minData」パラメータは、最小目標トルク範囲値が格納される「short」変数へのポインタです。
 * @param maxData 「maxData」パラメータは、最大目標トルク範囲値を格納する「short」変数へのポインタです。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::getMaxGoalTorqueRange(byte id, short *minData, short *maxData)
{
    byte rxData[4];
    unsigned short status = this->MemREAD(id, PMX::RamAddrList::MaxGoalTorqueMinRange, 4, rxData);

    byte rangeData[2];

    rangeData[0] = rxData[0];
    rangeData[1] = rxData[1];
    *minData = DataConv::bytesToInt16(rangeData);

    rangeData[0] = rxData[2];
    rangeData[1] = rxData[3];
    *maxData = DataConv::bytesToInt16(rangeData);

    //通信エラーの時は0x7FFFを入れておく
    if( (status & PMX::ComError::ErrorMask) != 0x0000 )
    {
        *minData = (short)PMX::ErrorUint16Data;
    }

    return status;
}



//
//MemWriteを利用したデータのset関数一覧
//

/**
 * 関数 `setPositionKpGain` は、PMX デバイスの特定の ID の位置 Kp ゲインを設定します。
 * 
 * @param id id パラメータは、位置 Kp ゲインが設定されているデバイスまたはコンポーネントの ID を表すバイトです。
 * @param data 「data」パラメータは、位置 Kp ゲインの値を表す符号なしの long 変数です。制御システムにおける位置制御の比例ゲインを設定するために使用されます。
 * @param writeOpt 「writeOpt」パラメータは、関数の書き込みオプションを指定するバイトです。これは、指定されたメモリ
 * アドレスにデータを書き込む方法を決定するために使用されます。特定の値とその意味は、関数の実装によって異なる場合があります。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setPositionKpGain(byte id, unsigned long data, byte writeOpt)
{
    return this->MemWRITEToUint32(id, PMX::RamAddrList::PositionKp, data, writeOpt);
}


/**
 * 関数 `setPositionKiGain` は、PMX デバイスの特定の ID の位置 Ki ゲインを設定します。
 * 
 * @param id id パラメータは、位置 Ki ゲインが設定されているデバイスまたはコンポーネントの ID を表すバイトです。
 * @param data 「データ」パラメータは、位置 Ki ゲインの値を表す符号なし長整数です。
 * @param writeOpt
 * 「writeOpt」パラメータは、メモリ書き込み操作の書き込みオプションを指定するバイトです。データをメモリに書き込む方法を決定するために使用されます。特定の値とその意味は、「MemWRITEToUint32」関数の実装によって異なる場合があります。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setPositionKiGain(byte id, unsigned long data, byte writeOpt)
{
    return this->MemWRITEToUint32(id, PMX::RamAddrList::PositionKi, data, writeOpt);
}


/**
 * PMX ベースの位置制御用の Kd ゲインを設定する機能です。
 * 
 * @param id id パラメータは、位置 Kd ゲインが設定されているデバイスまたはコンポーネントの ID を表すバイトです。
 * @param data 「データ」パラメータは、位置 Kd ゲインの値を表す符号なし長整数です。
 * @param writeOpt 「writeOpt」パラメータは、関数の書き込みオプションを指定するバイトです。これは、指定されたメモリ
 * アドレスにデータを書き込む方法を決定するために使用されます。特定の値とその意味は、PmxBace クラスのコードまたはドキュメント内で定義できます。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setPositionKdGain(byte id, unsigned long data, byte writeOpt)
{
    return this->MemWRITEToUint32(id, PMX::RamAddrList::PositionKd, data, writeOpt);
}

/**
 * 関数 `setPositionGain` は、特定のモーターの位置ゲイン値を設定します。
 * 
 * @param id id パラメータは、位置ゲインを設定するデバイスの ID を表すバイトです。
 * @param kpdata 「kpdata」パラメータは、位置比例ゲイン (Kp) の値を表す符号なし長整数です。
 * @param kidata パラメータ「kidata」は、位置制御ループの積分ゲイン値を表します。
 * @param kddata パラメータ「kddata」は、PID コントローラーの微分ゲイン値を表します。誤差の変化率に基づいて出力の変化率を制御するために使用されます。
 * @param writeOpt
 * 「writeOpt」パラメータは、メモリ書き込み操作の書き込みオプションを指定するバイトです。データをメモリに書き込む方法を決定するために使用されます。特定の値とその意味は、コード内の他の場所で定義される場合があります。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setPositionGain(byte id, unsigned long kpdata, unsigned long kidata, unsigned long kddata, byte writeOpt)
{
    byte txDataArray[12];
    byte pidData[4];
    DataConv::uint32ToBytes(kpdata, pidData);
    for(int i = 0; i < 4; i++){
        txDataArray[i] = pidData[i];
    }

    DataConv::uint32ToBytes(kidata, pidData);
    for(int i = 0; i < 4; i++){
        txDataArray[i+4] = pidData[i];
    }

    DataConv::uint32ToBytes(kddata, pidData);
    for(int i = 0; i < 4; i++){
        txDataArray[i+8] = pidData[i];
    }

    unsigned short status = this->MemWRITE(id, PMX::RamAddrList::PositionKp, txDataArray, 12, writeOpt);

    return status;

}

/**
 * 関数 `setPositionStretchGain` は、PMX オブジェクト内の特定の ID の位置ストレッチ ゲインを設定します。
 * 
 * @param id id パラメータは、位置ストレッチ ゲインが設定されているデバイスまたはコンポーネントの ID を表すバイトです。
 * @param data 「データ」パラメータは、位置ストレッチ ゲインの値を表す符号なし長整数です。
 * @param writeOpt 「writeOpt」パラメータは、setPositionStretchGain 関数の書き込みオプションを指定するバイトです。これは、指定された位置ストレッチ
 * ゲインにデータを書き込む方法を決定するために使用されます。特定の値とその意味は、関数の実装によって異なる場合があります。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setPositionStretchGain(byte id, unsigned long data, byte writeOpt)
{
    return this->MemWRITEToUint32(id, PMX::RamAddrList::PositionSt, data, writeOpt);
}

/**
 * この関数は、PMX Bace オブジェクト内の特定の ID の速度 Kp ゲインを設定します。
 * 
 * @param id id パラメータは、速度 Kp ゲインが設定されているデバイスまたはコンポーネントの ID を表すバイトです。
 * @param data 「data」パラメータは、指定されたレジスタに書き込まれる値を表す符号なしのlong変数です。
 * @param writeOpt
 * 「writeOpt」パラメータは、データの書き込みオプションを指定するバイトです。これは、指定されたアドレスにデータを書き込む方法を決定するために使用されます。特定の値とその意味は、「MemWRITEToUint32」関数の実装によって異なる場合があります。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setSpeedKpGain(byte id, unsigned long data, byte writeOpt)
{
    return this->MemWRITEToUint32(id, PMX::RamAddrList::SpeedKp, data, writeOpt);
}


/**
 * この関数は、PMX Bace オブジェクト内の特定の ID の速度 Ki ゲインを設定します。
 * 
 * @param id id パラメータは、速度 Ki ゲインが設定されているデバイスまたはコンポーネントの ID を表すバイトです。
 * @param data 「データ」パラメータは、速度 Ki ゲインの値を表す符号なし長整数です。
 * @param writeOpt
 * 「writeOpt」パラメータは、データの書き込みオプションを指定するバイトです。これは、指定されたアドレスにデータを書き込む方法を決定するために使用されます。特定の値とその意味は、「MemWRITEToUint32」関数の実装によって異なる場合があります。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setSpeedKiGain(byte id, unsigned long data, byte writeOpt)
{
    return this->MemWRITEToUint32(id, PMX::RamAddrList::SpeedKi, data, writeOpt);
}


/**
 * この関数は、PMX デバイスの速度制御のための Kd ゲインを設定します。
 * 
 * @param id id パラメータは、速度 Kd ゲインが設定されているデバイスまたはコンポーネントの ID を表すバイトです。
 * @param data 「データ」パラメータは、速度 Kd ゲインの値を表す符号なし長整数です。
 * @param writeOpt
 * 「writeOpt」パラメータは、データの書き込みオプションを指定するバイトです。これは、指定されたアドレスにデータを書き込む方法を決定するために使用されます。特定の値とその意味は、「MemWRITEToUint32」関数の実装によって異なる場合があります。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setSpeedKdGain(byte id, unsigned long data, byte writeOpt)
{
    return this->MemWRITEToUint32(id, PMX::RamAddrList::SpeedKd, data, writeOpt);
}


/**
 * 関数 `setSpeedGain` は、モーター コントローラーの速度ゲイン値を設定します。
 * 
 * @param id id パラメータはバイト型で、速度ゲインが設定されているデバイスの ID を表します。
 * @param kpdata 「kpdata」パラメータは「unsigned long」タイプで、速度制御アルゴリズムの比例ゲインの値を表します。
 * @param kidata パラメータ「kidata」は unsigned long 型で、速度制御アルゴリズムの積分ゲインの値を表します。
 * @param kddata パラメーター `kddata` はタイプ `unsigned long` であり、PID コントローラーの微分ゲインの値を表します。
 * @param writeOpt
 * パラメータ「writeOpt」は、メモリ書き込み操作の書き込みオプションを指定するバイトです。データをメモリに書き込む方法を決定するために使用されます。特定の値とその意味は、「MemWRITE」関数の実装によって異なります。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setSpeedGain(byte id, unsigned long kpdata, unsigned long kidata, unsigned long kddata, byte writeOpt)
{
    byte txDataArray[12];
    byte pidData[4];
    DataConv::uint32ToBytes(kpdata, pidData);
    for(int i = 0; i < 4; i++){
        txDataArray[i] = pidData[i];
    }

    DataConv::uint32ToBytes(kidata, pidData);
    for(int i = 0; i < 4; i++){
        txDataArray[i+4] = pidData[i];
    }

    DataConv::uint32ToBytes(kddata, pidData);
    for(int i = 0; i < 4; i++){
        txDataArray[i+8] = pidData[i];
    }

    unsigned short status = this->MemWRITE(id, PMX::RamAddrList::SpeedKp, txDataArray, 12, writeOpt);

    return status;
}

/**
 * この関数は、PMX デバイスの特定の ID の現在の比例ゲイン (Kp) を設定します。
 * 
 * @param id id パラメータは、Kp ゲインが設定されているデバイスまたはコンポーネントの ID を表すバイトです。
 * @param data 「data」パラメータは、設定する Kp ゲインの値を表す符号なし長整数です。
 * @param writeOpt
 * 「writeOpt」パラメータは、メモリ書き込み操作の書き込みオプションを指定するバイトです。データをメモリに書き込む方法を決定するために使用されます。特定の値とその意味は、PmxBace
 * クラスのコードまたはドキュメント内で定義できます。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setCurrentKpGain(byte id, unsigned long data, byte writeOpt)
{
    return this->MemWRITEToUint32(id, PMX::RamAddrList::CurrentKp, data, writeOpt);
}


/**
 * この関数は、PMX Bace オブジェクト内の特定の ID の現在の Ki ゲインを設定します。
 * 
 * @param id id パラメータは、Ki ゲインが設定されているデバイスまたはコンポーネントの ID を表すバイトです。
 * @param data 「データ」パラメータは、現在の Ki ゲインの値を表す符号なし長整数です。
 * @param writeOpt
 * 「writeOpt」パラメータは、メモリ書き込み操作の書き込みオプションを指定するバイトです。データをメモリに書き込む方法を決定するために使用されます。特定の値とその意味は、「MemWRITEToUint32」関数の実装によって異なる場合があります。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setCurrentKiGain(byte id, unsigned long data, byte writeOpt)
{
    return this->MemWRITEToUint32(id, PMX::RamAddrList::CurrentKi, data, writeOpt);
}


/**
 * この関数は、PMX デバイスの特定の ID の現在の Kd ゲインを設定します。
 * 
 * @param id id パラメータは、Kd ゲインが設定されているデバイスまたはコンポーネントの ID を表すバイトです。
 * @param data 「データ」パラメータは、Kd ゲインの値を表す符号なし長整数です。
 * @param writeOpt
 * 「writeOpt」パラメータは、データの書き込みオプションを指定するバイトです。これは、指定されたアドレスにデータを書き込む方法を決定するために使用されます。特定の値とその意味は、「MemWRITEToUint32」関数の実装によって異なる場合があります。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setCurrentKdGain(byte id, unsigned long data, byte writeOpt)
{
    return this->MemWRITEToUint32(id, PMX::RamAddrList::CurrentKd, data, writeOpt);
}


/**
 * 関数 `setCurrentGain` は、提供されたデータを使用して特定の ID の現在のゲイン値を設定します。
 * 
 * @param id id パラメータはバイト型で、ゲイン値が設定されているデバイスの ID を表します。
 * @param kpdata 「kpdata」パラメータは、PID コントローラの比例ゲイン (Kp) の値を表す符号なし長整数です。
 * @param kidata パラメータ「kidata」は、PID コントローラーの積分ゲインの値を表します。
 * @param kddata パラメータ「kddata」は、PID コントローラーの微分ゲイン (Kd) の値を表します。
 * @param writeOpt
 * パラメータ「writeOpt」は、メモリ書き込み操作の書き込みオプションを指定するバイトです。データをメモリに書き込む方法を制御するために使用されます。特定の値とその意味は、「MemWRITE」関数の実装によって異なります。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setCurrentGain(byte id, unsigned long kpdata, unsigned long kidata, unsigned long kddata, byte writeOpt)
{
    byte txDataArray[12];
    byte pidData[4];
    DataConv::uint32ToBytes(kpdata, pidData);
    for(int i = 0; i < 4; i++){
        txDataArray[i] = pidData[i];
    }

    DataConv::uint32ToBytes(kidata, pidData);
    for(int i = 0; i < 4; i++){
        txDataArray[i+4] = pidData[i];
    }

    DataConv::uint32ToBytes(kddata, pidData);
    for(int i = 0; i < 4; i++){
        txDataArray[i+8] = pidData[i];
    }

    unsigned short status = this->MemWRITE(id, PMX::RamAddrList::CurrentKp, txDataArray, 12, writeOpt);

    return status;

}



/**
 * 関数「setTorqueKpGain」は、PMX デバイスの特定の ID のトルク Kp ゲインを設定します。
 * 
 * @param id トルク Kp ゲインが設定されているデバイスまたはコンポーネントの ID。
 * @param data 「データ」パラメータは、トルク Kp ゲインの値を表す符号なし長整数です。
 * @param writeOpt 「writeOpt」パラメータは、データの書き込みオプションを指定するバイトです。これは、指定されたメモリ アドレスにデータを書き込む方法を決定するために使用されます。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setTorqueKpGain(byte id, unsigned long data, byte writeOpt)
{
    return this->MemWRITEToUint32(id, PMX::RamAddrList::TorqueKp, data, writeOpt);
}


/**
 * 関数「setTorqueKiGain」は、PMX デバイスの特定の ID のトルク Ki ゲインを設定します。
 * 
 * @param id id パラメータは、トルク Ki ゲインが設定されているデバイスまたはコンポーネントの ID を表すバイトです。
 * @param data 「data」パラメータは、トルク Ki ゲインの値を表す符号なし長整数です。
 * @param writeOpt
 * 「writeOpt」パラメータは、メモリ書き込み操作の書き込みオプションを指定するバイトです。データをメモリに書き込む方法を決定するために使用されます。特定の値とその意味は、「MemWRITEToUint32」関数の実装によって異なる場合があります。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setTorqueKiGain(byte id, unsigned long data, byte writeOpt)
{
    return this->MemWRITEToUint32(id, PMX::RamAddrList::TorqueKi, data, writeOpt);
}


/**
 * 関数「setTorqueKdGain」は、PMX デバイスの特定の ID の Torque Kd ゲインを設定します。
 * 
 * @param id id パラメータは、トルク Kd ゲインが設定されているデバイスまたはコンポーネントの ID を表すバイトです。
 * @param data 「データ」パラメータは、トルク Kd ゲインの値を表す符号なし長整数です。
 * @param writeOpt
 * 「writeOpt」パラメータは、データの書き込みオプションを指定するバイトです。これは、指定されたアドレスにデータを書き込む方法を決定するために使用されます。特定の値とその意味は、「MemWRITEToUint32」関数の実装によって異なる場合があります。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setTorqueKdGain(byte id, unsigned long data, byte writeOpt)
{
    return this->MemWRITEToUint32(id, PMX::RamAddrList::TorqueKd, data, writeOpt);
}


/**
 * 関数 `setTorqueGain` は、特定のモーターのトルク ゲイン値を設定します。
 * 
 * @param id トルクゲインを設定するデバイスのID。
 * @param kpdata `kpdata` パラメータは `unsigned long` 型で、トルク制御の比例ゲインの値を表します。
 * @param kidata パラメータ「kidata」は、トルク制御の積分ゲインの値である。これは unsigned long 型であり、32 ビットの符号なし整数値を保持できることを意味します。
 * @param kddata パラメータ「kddata」は、トルク制御アルゴリズムの微分ゲインの値を表します。
 * @param writeOpt `writeOpt` パラメータは、`MemWRITE`
 * 関数の書き込みオプションを指定するバイトです。データをメモリに書き込む方法を決定するために使用されます。特定の値とその意味は、`MemWRITE` 関数の実装によって異なります。
 * 
 * @return 操作のステータスを表す符号なしの short 値。
 */
unsigned short PmxBace::setTorqueGain(byte id, unsigned long kpdata, unsigned long kidata, unsigned long kddata, byte writeOpt)
{
    byte txDataArray[12];
    byte pidData[4];
    DataConv::uint32ToBytes(kpdata, pidData);
    for(int i = 0; i < 4; i++){
        txDataArray[i] = pidData[i];
    }

    DataConv::uint32ToBytes(kidata, pidData);
    for(int i = 0; i < 4; i++){
        txDataArray[i+4] = pidData[i];
    }

    DataConv::uint32ToBytes(kddata, pidData);
    for(int i = 0; i < 4; i++){
        txDataArray[i+8] = pidData[i];
    }

    unsigned short status = this->MemWRITE(id, PMX::RamAddrList::TorqueKp, txDataArray, 12, writeOpt);

    return status;

}

/**
 * 関数「setCenterOffset」は、PMX オブジェクト内の特定の ID の中心オフセット値を設定します。
 * 
 * @param id 「id」パラメータは、中心オフセットが設定されているオブジェクトまたは要素の ID を表すバイトです。
 * @param offsetData offsetData パラメータは、設定する中心オフセット値を表す符号付き 16 ビット整数です。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setCenterOffset(byte id,short offsetData)
{
    return this->MemWRITEToInt16(id, PMX::RamAddrList::CenterOffset, offsetData);
}

/**
 * 関数「setCloneReverse」は、PMX オブジェクト内の特定の ID のクローン リバース値を設定します。
 * 
 * @param id 「id」パラメータは、PmxBace オブジェクトの ID を表すバイトです。これは、メソッドが呼び出される特定のオブジェクトを識別するために使用されます。
 * @param data 「data」パラメータは、クローンリバース機能に設定する値を表すバイト値です。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setCloneReverse(byte id, byte data)
{
    return this->MemWRITEToByte(id, PMX::RamAddrList::CloneReverse, data);
}

/**
 * 関数 `setMinVoltageLimit` は、PMX Bace システムの特定の ID の最小電圧制限と制限電力を設定します。
 * 
 * @param id id パラメータは、最小電圧制限を設定するデバイスの ID を表すバイトです。
 * @param minVol minVol パラメータは、設定する最小電圧制限値です。これは unsigned short 型であり、0 ～ 65535 の値を保持できることを意味します。
 * @param limPower 「limPower」パラメータは unsigned short 型で、制限電力値を表します。
 * @param writeOpt 「writeOpt」パラメータは、メモリ書き込み操作の書き込みオプションを指定するバイトです。データをメモリに書き込む方法を決定するために使用されます。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setMinVoltageLimit(byte id,unsigned short minVol, unsigned short limPower ,byte writeOpt)
{
    byte txDataArray[4];
    DataConv::uint16ToBytes(minVol, txDataArray);
    DataConv::uint16ToBytes(limPower, &(txDataArray[2]));

    return this->MemWRITE(id, PMX::RamAddrList::MinVoltageLimit, txDataArray, 4, writeOpt);
}

/**
 * この関数は、PMX Bace システムの特定の ID の最大電圧制限と電力制限を設定します。
 * 
 * @param id id パラメータは、最大電圧制限を設定するデバイスの ID を表すバイトです。
 * @param maxVol maxVol パラメータは、最大電圧制限を表す符号なし short です。
 * @param limPower パラメータ「limPower」は限界電力値を表す。
 * @param writeOpt
 * 「writeOpt」パラメータは、メモリ書き込み操作の書き込みオプションを指定するバイトです。データをメモリに書き込む方法を決定するために使用されます。特定の値とその意味は、使用されているコードまたはライブラリのコンテキストで定義できます。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setMaxVoltageLimit(byte id,unsigned short maxVol, unsigned short limPower ,byte writeOpt)
{
    byte txDataArray[4];
    DataConv::uint16ToBytes(maxVol, txDataArray);
    DataConv::uint16ToBytes(limPower, &(txDataArray[2]));

    return this->MemWRITE(id, PMX::RamAddrList::MaxVoltageLimit, txDataArray, 4, writeOpt);
}

/**
 * 関数 `setCurrentLimit` は、PMX Bace システムの特定の ID の電流制限と電力制限を設定します。
 * 
 * @param id id パラメータは、電流制限を設定するデバイスの ID を表すバイトです。
 * @param maxCur 指定した ID に設定する最大電流制限 (ミリアンペア (mA) 単位)。
 * @param limPower 「limPower」パラメータは、電流の電力制限を表す符号なし short です。
 * @param writeOpt
 * 「writeOpt」パラメータは、メモリ書き込み操作の書き込みオプションを指定するバイトです。データをメモリに書き込む方法を決定するために使用されます。特定の値とその意味は、使用されているコードまたはライブラリのコンテキストで定義できます。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setCurrentLimit(byte id, short maxCur, unsigned short limPower ,byte writeOpt)
{
    byte txDataArray[4];
    DataConv::int16ToBytes(maxCur, txDataArray);
    DataConv::uint16ToBytes(limPower, &(txDataArray[2]));

    return this->MemWRITE(id, PMX::RamAddrList::CurrentLimit, txDataArray, 4, writeOpt);
}

/**
 * 関数 `setMotorTempLimit` は、特定のモーターのモーター温度制限と電力制限を設定します。
 * 
 * @param id id パラメータは、モーターの ID を表すバイトです。
 * @param motorTemp モーターの温度 (摂氏)。
 * @param limPower パラメータ「limPower」は、モーターの電力制限を表す符号なし short です。
 * @param writeOpt
 * 「writeOpt」パラメータは、メモリ書き込み操作の書き込みオプションを指定するバイトです。データをメモリに書き込む方法を決定するために使用されます。特定の値とその意味は、使用されているコードまたはライブラリのコンテキストで定義できます。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setMotorTempLimit(byte id, short motorTemp, unsigned short limPower ,byte writeOpt)
{
    byte txDataArray[4];
    DataConv::int16ToBytes(motorTemp, txDataArray);
    DataConv::uint16ToBytes(limPower, &(txDataArray[2]));

    return this->MemWRITE(id, PMX::RamAddrList::MotorTempLimit, txDataArray, 4, writeOpt);
}

/**
 * 関数 `setCpuTempLimit` は、特定の ID の CPU 温度制限と電力制限を設定します。
 * 
 * @param id id パラメータは、CPU 温度制限を設定するデバイスの ID を表すバイトです。
 * @param cpuTemp cpuTemp パラメータは、必要な CPU 温度制限を表す短い整数です。
 * @param limPower 「limPower」パラメータは、CPU 温度の電力制限を表す符号なし short です。
 * @param writeOpt
 * 「writeOpt」パラメータは、メモリ書き込み操作の書き込みオプションを指定するバイトです。データをメモリに書き込む方法を決定するために使用されます。特定の値とその意味は、使用されているコードまたはライブラリのコンテキストで定義できます。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setCpuTempLimit(byte id, short cpuTemp, unsigned short limPower ,byte writeOpt)
{
    byte txDataArray[4];
    DataConv::int16ToBytes(cpuTemp, txDataArray);
    DataConv::uint16ToBytes(limPower, &(txDataArray[2]));

    return this->MemWRITE(id, PMX::RamAddrList::CpuTempLimit, txDataArray, 4, writeOpt);
}

/**
 * この関数は、特定のモーターの位置制限、電力制限、および書き込みオプションを設定します。
 * 
 * @param id PMX デバイスの ID。
 * @param cwPos cwPos パラメータは、指定された ID を持つ指定されたモーターの時計回りの位置制限を表す short 整数です。
 * @param ccwPos ccwPos パラメータは、指定された ID を持つ特定のモーターの反時計回りの位置制限を表す short 整数です。
 * @param limPower パラメータ「limPower」は、モータの位置制御の限界電力を表します。これは unsigned short データ型であり、0 ～ 65535
 * の値を保持できることを意味します。
 * @param writeOpt
 * パラメータ「writeOpt」は、メモリ書き込み操作の書き込みオプションを指定するバイトです。データをメモリに書き込む方法を決定するために使用されます。特定の値とその意味は、ドキュメントまたは「MemWRITE」関数のコード実装で定義される場合があります。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setPositionLimit(byte id, short cwPos, short ccwPos, unsigned short limPower ,byte writeOpt)
{
    byte txDataArray[6];
    DataConv::int16ToBytes(cwPos, txDataArray);
    DataConv::int16ToBytes(ccwPos, &(txDataArray[2]));
    DataConv::uint16ToBytes(limPower, &(txDataArray[4]));

    return this->MemWRITE(id, PMX::RamAddrList::CwPositionLimit, txDataArray, 6, writeOpt);
}

/**
 * この関数は、PMX Bace オブジェクトの特定の ID の最大目標速度を設定します。
 * 
 * @param id id パラメータは、最大目標速度が設定されているモーターまたはデバイスの ID を表すバイトです。
 * @param maxGoalSpd maxGoalSpd パラメーターは、最大ゴール速度値を表す short 整数です。
 * @param writeOpt
 * 「writeOpt」パラメータは、メモリ書き込み操作の書き込みオプションを指定するバイトです。データをメモリに書き込む方法を決定するために使用されます。特定の値とその意味は、使用しているコードまたはライブラリのコンテキスト内で定義される場合があります。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setMaxGoalSpeed(byte id, short maxGoalSpd, byte writeOpt)
{
    return this->MemWRITEToInt16(id, PMX::RamAddrList::MaxGoalSpeed, maxGoalSpd, writeOpt);
}

/**
 * この関数は、PMX Bace オブジェクト内の特定の ID の最大目標電流を設定します。
 * 
 * @param id id パラメータは、最大目標電流を設定するデバイスの ID を表すバイトです。
 * @param maxGoalCur maxGoalCur パラメーターは、現在の最大目標値を表す short 整数です。
 * @param writeOpt 「writeOpt」パラメータは、メモリ書き込み操作の書き込みオプションを指定するバイトです。データをメモリに書き込む方法を決定するために使用されます。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setMaxGoalCurrent(byte id, short maxGoalCur, byte writeOpt)
{
    return this->MemWRITEToInt16(id, PMX::RamAddrList::MaxGoalCurrent, maxGoalCur, writeOpt);
}

/**
 * この関数は、PMX デバイスの特定の ID の最大目標トルクを設定します。
 * 
 * @param id id パラメータは、最大目標トルクが設定されているモーターまたはデバイスの ID を表すバイトです。
 * @param maxGoalTrq maxGoalTrq パラメーターは、最大目標トルク値を表す short 整数です。
 * @param writeOpt 「writeOpt」パラメータは、メモリ書き込み操作の書き込みオプションを指定するバイトです。データをメモリに書き込む方法を決定するために使用されます。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setMaxGoalTorque(byte id, short maxGoalTrq, byte writeOpt)
{
    return this->MemWRITEToInt16(id, PMX::RamAddrList::MaxGoalTorque, maxGoalTrq, writeOpt);
}

/**
 * 関数「setTotalPowerRate」は、PMX システム内の特定の ID の合計電力レートを設定します。
 * 
 * @param id id パラメータは、合計電力レートが設定されているデバイスまたはコンポーネントの識別子を表すバイトです。
 * @param rate 「rate」パラメータは符号なし short であり、0 ～ 65535 の値を保持できることを意味します。これは、特定の ID に設定される合計電力レート値を表します。
 * @param writeOpt 「writeOpt」パラメータは、メモリ書き込み操作の書き込みオプションを指定するバイトです。データをメモリに書き込む方法を決定するために使用されます。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setTotalPowerRate(byte id,unsigned short rate, byte writeOpt)
{
    return this->MemWRITEToUint16(id, PMX::RamAddrList::TotalPowerRate, rate, writeOpt);
}

/**
 * 関数 `setLockDetect` は、指定された ID のロック検出時間、電力、および出力電力を設定します。
 * 
 * @param id 「id」パラメータは、ロック検出パラメータを設定するデバイスの ID を表すバイトです。
 * @param time 「time」パラメータは、ロック検出時間を表す符号なし short です。ロック検出がアクティブになる期間を指定します。
 * @param power 「power」パラメータは、設定される電力値を表す符号なし short です。
 * @param outputPower 「outputPower」パラメータは、必要な出力電力値を表す符号なし short です。
 * @param writeOpt
 * パラメータ「writeOpt」はバイト型で、メモリ書き込み操作の書き込みオプションを指定するために使用されます。おそらく、データをメモリに書き込む方法を制御するオプションです。特定の値とその意味は、「MemWRITE」関数の実装によって異なります。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setLockDetect(byte id, unsigned short time, unsigned short power, unsigned short outputPower, byte writeOpt)
{
    byte txDataArray[6];
    DataConv::uint16ToBytes(time, txDataArray);
    DataConv::uint16ToBytes(power, &(txDataArray[2]));
    DataConv::uint16ToBytes(outputPower, &(txDataArray[4]));

    return this->MemWRITE(id, PMX::RamAddrList::LockDetectTime, txDataArray, 6, writeOpt);
}


/**
 * 指定された ID の位置制御などの制御モードを設定します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] controlMode モータに設定する制御モード(ControlModeを参照)
 * @param [in] writeOpt TolqueOn状態でも強制的に書き込むか設定します。0:強制書き込みしない、1:強制書き込みする(デフォルトは0)
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::setControlMode(byte id, byte controlMode,byte writeOpt)
{
    return this->MemWRITEToByte(id, PMX::RamAddrList::ControlMode, controlMode, writeOpt);
}

/**
 * 指定したIDのmotorREADやmotorWRITEから返信される応答モードを設定します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] receiveOpt 応答モード ReceiveDataOptionを参照
 * @param [in] writeOpt TolqueOn状態でも強制的に書き込むか設定します。0:強制書き込みしない、1:強制書き込みする(デフォルトは0)
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::setMotorReceive(byte id, byte receiveMode,byte writeOpt)
{
    return this->MemWRITEToByte(id, PMX::RamAddrList::MotorReceiveData, receiveMode, writeOpt);
}

/**
 * 関数「setTrajectory」は、PMX オブジェクト内の特定の ID の軌跡データを設定します。
 * 
 * @param id id パラメータは、軌跡の ID を表すバイトです。どの軌道を設定するかを識別するために使用されます。
 * @param trajecrotyData trajecrotyData パラメータは、設定する軌跡データを表すバイトです。
 * @param writeOpt
 * 「writeOpt」パラメータは、軌跡データの書き込みオプションを指定するバイトです。データをメモリに書き込む方法を決定するために使用されます。特定の値とその意味は、「MemWRITEToByte」関数の実装によって異なります。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setTrajectory(byte id, byte trajecrotyData, byte writeOpt)
{
    return this->MemWRITEToByte(id, PMX::RamAddrList::Trajectory, trajecrotyData, writeOpt);
}



//
//MotorWriteを利用したデータのset関数一覧
//

/**
 * この関数は、指定された ID を持つモーターのトルクを設定します。
 * 
 * @param id 「id」パラメータは、トルクを設定するモーターの ID です。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setMotorTorqueOn(byte id)
{
    return this->MotorWRITE(id, PMX::TorqueSwitchType::TorqueOn);
}

/**
 * この関数は、PMX ベースの特定のモーターに対してモーター トルクを設定します。
 * 
 * @param id モーターのID。
 * @param receiveMode ｒｅｃｅｉｖｅＭｏｄｅパラメータは、モータトルクを受信するモードを指定するバイトである。
 * @param receiveData パラメータ「receiveData」は、長さ 8 の長い整数の配列です。これは、関数にデータを渡すために使用されます。
 * @param controlMode 制御モードパラメータは、モータトルクを設定するモードを指定します。バイト型です。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setMotorTorqueOn(byte id,byte receiveMode, long receiveData[8], byte controlMode)
{
    return this->MotorWRITE(id, PMX::TorqueSwitchType::TorqueOn, receiveMode, receiveData, controlMode);
}


/**
 * 指定した ID のモータをフリーにし、ステータス、モータのON/OFF、受信データを取得します。n * 
 * @param [in] id PMXサーボモータのID番号
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::setMotorFree(byte id)
{
    return this->MotorWRITE(id, PMX::TorqueSwitchType::Free);
}


/**
 * 指定した ID のモータをフリーにし、ステータス、モータのON/OFF、受信データを取得します。
 * 
 * @param [in] id PMXサーボモータのID番号
 * @param [in] receiveMode PMXで設定した応答モード
 * @param [in] receiveData MotorWRITEで読み取ったデータ配列[位置,速度,電流,トルク,PWM,モータ温度,CPU温度,電圧](合計8項目)(配列にデータが代入されます)
 * @param [in] controlMode モータに設定する制御モード(ControlModeを参照)
 * 
 * @return status (Uint16) 通信の状態とステータス(送信結果(PMX::ComError参照) + PMXのstatus)
 */
unsigned short PmxBace::setMotorFree(byte id,byte receiveMode, long receiveData[8], byte controlMode)
{
    return this->MotorWRITE(id, PMX::TorqueSwitchType::Free, receiveMode, receiveData, controlMode);
}


/**
 * 関数「setMotorBrake」は、モーターのトルクスイッチタイプを「Brake」に設定します。
 * 
 * @param id 「id」パラメータは、モーターの ID を表すバイトです。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setMotorBrake(byte id)
{
    return this->MotorWRITE(id, PMX::TorqueSwitchType::Brake);
}


/**
 * 関数「setMotorBrake」は、PMX ベースのモーターのブレーキ モードを設定します。
 * 
 * @param id モーターのID。
 * @param receiveMode acceptMode パラメータは、モータ ブレーキを設定するモードを指定するバイトです。
 * @param receiveData acceptData パラメータは、長さが 8 の長い整数の配列です。データを setMotorBrake 関数に渡すために使用されます。
 * @param controlMode 制御モードにより、モーターがどのように制御されるかが決まります。アプリケーションの特定の要件に応じて、異なる値に設定できます。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setMotorBrake(byte id,byte receiveMode, long receiveData[8], byte controlMode)
{
    return this->MotorWRITE(id, PMX::TorqueSwitchType::Brake, receiveMode, receiveData, controlMode);
}   


/**
 * 関数「setMotorHold」は、モーターのトルクスイッチタイプを「Hold」に設定します。
 * 
 * @param id 「id」パラメータは、モーターの ID を表すバイトです。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setMotorHold(byte id)
{
    return this->MotorWRITE(id, PMX::TorqueSwitchType::Hold);
}

/**
 * 関数「setMotorHold」は、モーターをホールドトルクモードに設定します。
 * 
 * @param id モーターのID。
 * @param receiveMode acceptMode パラメータは、モーターがデータを受信するモードを指定するバイトです。
 * @param receiveData acceptData パラメータは、長さが 8 の long 整数の配列です。データを setMotorHold 関数に渡すために使用されます。
 * @param controlMode 制御モードパラメータは、モーターを制御するモードを指定します。これは、使用される制御戦略を決定するバイト値です。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setMotorHold(byte id,byte receiveMode, long receiveData[8], byte controlMode)
{
    return this->MotorWRITE(id, PMX::TorqueSwitchType::Hold, receiveMode, receiveData, controlMode);
}

/**
 * この関数は、指定された ID を持つモーターの位置を設定します。
 * 
 * @param id id パラメータは、モーターの ID を表すバイトです。
 * @param pos 「pos」パラメータのタイプは「short」です。これは、符号付き 16 ビット整数であることを意味します。これは、指定された ID
 * を持つモーターが設定される必要がある望ましい位置を表します。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setPosition(byte id, short pos)
{
    return this->MotorWRITESingle(id, (long)pos);
}

/**
 * 関数 `setPosition` はモーターの位置を設定し、unsigned short 値を返します。
 * 
 * @param id id パラメータは、モーターの ID を表すバイトです。
 * @param pos 「pos」パラメータは「short」タイプで、指定された ID を持つモーターに設定する必要な位置を表します。
 * @param receiveMode acceptMode パラメータは、データを受信するモードを指定するバイトです。データがどのように処理または解釈されるかを決定するために使用されます。
 * @param receiveData acceptData パラメータは、長さが 8 の long 整数の配列です。これは、通信プロセス中に受信したデータを格納するために使用されます。
 * 
 * @return 符号なしの short 値。
 */
unsigned short PmxBace::setPosition(byte id, short pos, byte receiveMode, long receiveData[8])
{
    return this->MotorWRITESingle(id, (long)pos, receiveMode, receiveData);
}



